/****************************************************************************
 *                   ^     +----- |  / ^     ^ |     | +-\                  *
 *                  / \    |      | /  |\   /| |     | |  \                 *
 *                 /   \   +---   |<   | \ / | |     | |  |                 *
 *                /-----\  |      | \  |  v  | |     | |  /                 *
 *               /       \ |      |  \ |     | +-----+ +-/                  *
 ****************************************************************************
 * AFKMud Copyright 1997-2003 by Roger Libiez (Samson),                     *
 * Levi Beckerson (Whir), Michael Ward (Tarl), Erik Wolfe (Dwip),           *
 * Cameron Carroll (Cam), Cyberfox, Karangi, Rathian, Raine, and Adjani.    *
 * All Rights Reserved.                                                     *
 *                                                                          *
 * Original SMAUG 1.4a written by Thoric (Derek Snider) with Altrag,        *
 * Blodkai, Haus, Narn, Scryn, Swordbearer, Tricops, Gorog, Rennard,        *
 * Grishnakh, Fireblade, and Nivek.                                         *
 *                                                                          *
 * Original MERC 2.1 code by Hatchet, Furey, and Kahn.                      *
 *                                                                          *
 * Original DikuMUD code by: Hans Staerfeldt, Katja Nyboe, Tom Madsen,      *
 * Michael Seifert, and Sebastian Hammer.                                   *
 ****************************************************************************
 *                           Deity handling module                          *
 ****************************************************************************/

/* Put together by Rennard for Realms of Despair.  Brap on... */

#include "mud.h"
#include "clans.h"
#include "deity.h"

extern int astral_target;

bool check_pets( CHAR_DATA *ch, MOB_INDEX_DATA *pet );

DEITY_DATA *first_deity;
DEITY_DATA *last_deity;

void free_deity( DEITY_DATA *deity )
{
   UNLINK( deity, first_deity, last_deity, next, prev );
   STRFREE( deity->name );
   DISPOSE( deity->deitydesc );
   DISPOSE( deity->filename );
   DISPOSE( deity );
   return;
}

void free_deities( void )
{
   DEITY_DATA *deity, *deity_next;

   for( deity = first_deity; deity; deity = deity_next )
   {
	deity_next = deity->next;
      free_deity( deity );
   }
    return;
 }

/* Get pointer to deity structure from deity name */
DEITY_DATA *get_deity( char *name )
{
     DEITY_DATA *deity;
   for( deity = first_deity; deity; deity = deity->next )
	if( !str_cmp( name, deity->name ) )
	  return deity;
	return NULL;
}

void write_deity_list( void )
{
     DEITY_DATA *tdeity;
     FILE *fpout;
     char filename[256];

   snprintf( filename, 256, "%s%s", DEITY_DIR, DEITY_LIST );
     fpout = fopen( filename, "w" );
   if( !fpout )
	bug( "%s", "FATAL: cannot open deity.lst for writing!" );
     else
     {
      for( tdeity = first_deity; tdeity; tdeity = tdeity->next )
	  fprintf( fpout, "%s\n", tdeity->filename );
       fprintf( fpout, "%s", "$\n" );
       FCLOSE( fpout );
     }
}

/* Save a deity's data to its data file */
/* Modified to include deity recallroom for supplication - Samson 4-13-98 */
void save_deity( DEITY_DATA *deity )
{
     FILE *fp;
     char filename[256];

   if( !deity )
     {
	bug( "%s", "save_deity: null deity pointer!" );
	return;
     }

   if( !deity->filename || deity->filename[0] == '\0' )
     {
	bug( "save_deity: %s has no filename", deity->name );
	return;
     }

   snprintf( filename, 256, "%s%s", DEITY_DIR, deity->filename );

   if( ( fp = fopen( filename, "w" ) ) == NULL )
     {
	bug( "%s", "save_deity: fopen" );
	perror( filename );
     }
     else
     {
	fprintf( fp, "%s", "#DEITY\n" );
	fprintf( fp, "Filename		%s~\n",	deity->filename );
	fprintf( fp, "Name		%s~\n",	deity->name	);
	fprintf( fp, "Description	%s~\n",	deity->deitydesc );
	fprintf( fp, "Alignment		%d\n",	deity->alignment );
	fprintf( fp, "Worshippers	%d\n",	deity->worshippers );
	fprintf( fp, "Flee		%d\n",	deity->flee );
	fprintf( fp, "Flee_npcrace	%d\n",	deity->flee_npcrace );
	fprintf( fp, "Flee_npcfoe	%d\n",	deity->flee_npcfoe );
	fprintf( fp, "Kill		%d\n",	deity->kill );
	fprintf( fp, "Kill_npcrace	%d\n",	deity->kill_npcrace );
	fprintf( fp, "Kill_npcfoe	%d\n",	deity->kill_npcfoe );
	fprintf( fp, "Kill_magic	%d\n",	deity->kill_magic );
	fprintf( fp, "Sac		%d\n",	deity->sac );
	fprintf( fp, "Bury_corpse	%d\n",	deity->bury_corpse );
	fprintf( fp, "Aid_spell		%d\n",	deity->aid_spell );
	fprintf( fp, "Aid		%d\n",	deity->aid );
	fprintf( fp, "Steal		%d\n",	deity->steal );
	fprintf( fp, "Backstab		%d\n",	deity->backstab );
	fprintf( fp, "Die		%d\n",	deity->die );
	fprintf( fp, "Die_npcrace	%d\n",	deity->die_npcrace );
	fprintf( fp, "Die_npcfoe	%d\n",	deity->die_npcfoe );
	fprintf( fp, "Spell_aid		%d\n",	deity->spell_aid );
	fprintf( fp, "Dig_corpse	%d\n",	deity->dig_corpse );
	fprintf( fp, "Scorpse		%d\n",	deity->scorpse );
	fprintf( fp, "Savatar		%d\n",	deity->savatar );
	fprintf( fp, "Smount		%d\n",	deity->smount ); /* Added by Tarl 24 Feb 02 */
    fprintf( fp, "Sminion		%d\n",	deity->sminion ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Sdeityobj		%d\n",	deity->sdeityobj );
	fprintf( fp, "Sdeityobj2	%d\n",	deity->sdeityobj2 ); /* Added by Tarl 02 Mar 02 */
	fprintf( fp, "Srecall		%d\n",	deity->srecall );
	fprintf( fp, "Race		%d\n",	deity->race );
	fprintf( fp, "Class		%d\n",	deity->Class );
	fprintf( fp, "Class2	%d\n",	deity->Class2 ); /* Added by Tarl 25 Feb 02 */
	fprintf( fp, "Class3	%d\n",	deity->Class3 ); /* Added by Tarl 25 Feb 02 */
	fprintf( fp, "Class4	%d\n",	deity->Class4 ); /* Added by Tarl 25 Feb 02 */
	fprintf( fp, "Element		%d\n",	deity->element );
        fprintf( fp, "Element2          %d\n",  deity->element2 ); /* Added by Tarl 25 Feb 02 */
        fprintf( fp, "Element3          %d\n",  deity->element3 ); /* Added by Tarl 25 Feb 02 */
	fprintf( fp, "Sex		%d\n",	deity->sex );
	fprintf( fp, "Affected		%s\n",	print_bitvector(&deity->affected) );
	fprintf( fp, "Affected2         %s\n",  print_bitvector(&deity->affected2) ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Affected3         %s\n",  print_bitvector(&deity->affected3) ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Npcrace		%d\n",	deity->npcrace );
	fprintf( fp, "Npcfoe		%d\n",	deity->npcfoe );
	fprintf( fp, "Suscept		%d\n",	deity->suscept );
    fprintf( fp, "Suscept2          %d\n",  deity->suscept2 ); /* Added by Tarl 24 Feb 02 */
    fprintf( fp, "Suscept3          %d\n",  deity->suscept3 ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Race2			%d\n",	deity->race2 ); 
	fprintf( fp, "Race3			%d\n",	deity->race3 ); /* Added by Tarl 25 Feb 02 */
	fprintf( fp, "Race4			%d\n",	deity->race4 ); /* Added by Tarl 25 Feb 02 */
	fprintf( fp, "Susceptnum	%d\n",  deity->susceptnum );
    fprintf( fp, "Susceptnum2       %d\n",  deity->susceptnum2 ); /* Added by Tarl 24 Feb 02 */
    fprintf( fp, "Susceptnum3       %d\n",  deity->susceptnum3 ); /* Added by Tarl 24 Feb 02 */
    fprintf( fp, "Elementnum        %d\n",  deity->elementnum );
    fprintf( fp, "Elementnum2       %d\n",  deity->elementnum2 ); /* Added by Tarl 24 Feb 02 */ 
	fprintf( fp, "Elementnum3       %d\n",  deity->elementnum3 ); /* Added by Tarl 24 Feb 02 */
    fprintf( fp, "Affectednum       %d\n",  deity->affectednum );
    fprintf( fp, "Affectednum2      %d\n",  deity->affectednum2 ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Affectednum3	%d\n",	deity->affectednum3 ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Spell1		%d\n", deity->spell1 ); /* Added by Tarl 24 Mar 02 */
	fprintf( fp, "Spell2		%d\n", deity->spell2 ); /* Added by Tarl 24 Mar 02 */
	fprintf( fp, "Spell3		%d\n", deity->spell3 ); /* Added by Tarl 24 Mar 02 */
	fprintf( fp, "Sspell1		%d\n", deity->sspell1); /* Added by Tarl 24 Mar 02 */
	fprintf( fp, "Sspell2		%d\n", deity->sspell2); /* Added by Tarl 24 Mar 02 */
	fprintf( fp, "Sspell3		%d\n", deity->sspell3); /* Added by Tarl 24 Mar 02 */
	fprintf( fp, "Objstat		%d\n",	deity->objstat	);
	fprintf( fp, "Recallroom	%d\n",	deity->recallroom ); /* Samson */
	fprintf( fp, "Avatar		%d\n",	deity->avatar ); /* Restored by Samson */
	fprintf( fp, "Mount			%d\n",	deity->mount ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Minion		%d\n",	deity->minion ); /* Added by Tarl 24 Feb 02 */
	fprintf( fp, "Deityobj		%d\n",	deity->deityobj	); /* Restored by Samson */
	fprintf( fp, "Deityobj2		%d\n",	deity->deityobj2	); /* Added by Tarl 02 Mar 02 */
	fprintf( fp, "%s", "End\n\n" );
	fprintf( fp, "%s", "#END\n" );
	FCLOSE( fp );
     }
     return;
}

/* Read in actual deity data */

/* Modified to include new recallroom for supplication - Samson 4-13-98 */
void fread_deity( DEITY_DATA *deity, FILE *fp )
{
   const char *word;
     bool fMatch;

   for( ; ; )
     {
	word   = feof( fp ) ? "End" : fread_word( fp );
	fMatch = FALSE;

	switch ( UPPER(word[0]) )
	{
	case '*':
	    fMatch = TRUE;
	    fread_to_eol( fp );
	    break;

	case 'A':
	    KEY( "Affected",	deity->affected,	fread_bitvector( fp ) );
        KEY( "Affected2",   deity->affected2,	fread_bitvector( fp ) ); /* Added by Tarl 24 Feb 02 */
	    KEY( "Affected3",	deity->affected3,	fread_bitvector( fp ) ); /* Added by Tarl 24 Feb 02 */
	    KEY( "Affectednum",	deity->affectednum,	fread_number( fp ) );
	    KEY( "Affectednum2",deity->affectednum2,	fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
	    KEY( "Affectednum3",deity->affectednum3,	fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
	    KEY( "Aid",		deity->aid,		fread_number( fp ) );
	    KEY( "Aid_spell",	deity->aid_spell,	fread_number( fp ) );
	    KEY( "Alignment",	deity->alignment,	fread_number( fp ) );
	    KEY( "Avatar",	deity->avatar,	fread_number( fp ) ); /* Restored by Samson */
	    break;	

	case 'B':
	    KEY( "Backstab",	deity->backstab,	fread_number( fp ) );
	    KEY( "Bury_corpse",	deity->bury_corpse,	fread_number( fp ) );
	    break;

	case 'C':
	      KEY( "Class",	deity->Class,		fread_number( fp ) );	
	      KEY( "Class2",	deity->Class2,		fread_number( fp ) ); /* Added by Tarl 25 Feb 02 */
	      KEY( "Class3",	deity->Class3,		fread_number( fp ) ); /* Added by Tarl 25 Feb 02 */
	      KEY( "Class4",	deity->Class4,		fread_number( fp ) ); /* Added by Tarl 25 Feb 02 */
	    break;

	case 'D':
	    KEY( "Deityobj",	deity->deityobj,	fread_number( fp ) ); /* Restored by Samson */
	    KEY( "Deityobj2",	deity->deityobj2,	fread_number( fp ) ); /* Added by Tarl 02 Mar 02 */
	      KEY( "Description",	deity->deitydesc,	fread_string_nohash( fp ) );
	    KEY( "Die",		deity->die,		fread_number( fp ) );
	    KEY( "Die_npcrace",	deity->die_npcrace,	fread_number( fp ) );
	    KEY( "Die_npcfoe",	deity->die_npcfoe,	fread_number( fp ) );
	    KEY( "Dig_corpse",	deity->dig_corpse,	fread_number( fp ) );
	    break;

	case 'E':
	      if( !str_cmp( word, "End" ) )
	    {
		return;
	    }
	    KEY( "Element",	deity->element,		fread_number( fp ) );
        KEY( "Element2",    deity->element2,        fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
        KEY( "Element3",    deity->element3,        fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
	    KEY( "Elementnum",	deity->elementnum,	fread_number( fp ) );
        KEY( "Elementnum2", deity->elementnum2,     fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
        KEY( "Elementnum3",  deity->elementnum3,     fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
	    break;

	case 'F':
	    KEY( "Filename",	deity->filename,	fread_string_nohash( fp ) );
	    KEY( "Flee",	deity->flee,		fread_number( fp ) ); 
	    KEY( "Flee_npcrace",deity->flee_npcrace,	fread_number( fp ) );
	    KEY( "Flee_npcfoe",	deity->flee_npcfoe,	fread_number( fp ) );
	    break;

	case 'K':
	    KEY( "Kill",	deity->kill,		fread_number( fp ) );
	    KEY( "Kill_npcrace",deity->kill_npcrace,	fread_number( fp ) );
	    KEY( "Kill_npcfoe",	deity->kill_npcfoe,	fread_number( fp ) );
	    KEY( "Kill_magic",	deity->kill_magic,	fread_number( fp ) );
	    break;
	
	case 'M':
		KEY( "Minion",	deity->minion,	fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
		KEY( "Mount",	deity->mount,	fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
		break;

	case 'N':
	    KEY( "Name",	deity->name,		fread_string( fp ) );
	    KEY( "Npcfoe",	deity->npcfoe,		fread_number( fp ) );
	    KEY( "Npcrace",	deity->npcrace,		fread_number( fp ) );
	    break;

	case 'O':
	    KEY( "Objstat",	deity->objstat,		fread_number( fp ) );
	    break;

	case 'R':
	    KEY( "Race",	deity->race,		fread_number( fp ) );
	    KEY( "Race2",	deity->race2,		fread_number( fp ) );
	    KEY( "Race3",	deity->race3,		fread_number( fp ) ); /* Added by Tarl 25 Feb 02 */
	    KEY( "Race4",	deity->race4,		fread_number( fp ) ); /* Added by Tarl 25 Feb 02 */
	    KEY( "Recallroom", deity->recallroom, fread_number( fp ) );
	    break;

	case 'S':
	    KEY( "Sac",		deity->sac,		fread_number( fp ) );
	    KEY( "Savatar",	deity->savatar,		fread_number( fp ) );
	    KEY( "Scorpse",	deity->scorpse,		fread_number( fp ) );
	    KEY( "Sdeityobj",	deity->sdeityobj,	fread_number( fp ) );
	    KEY( "Sdeityobj2",	deity->sdeityobj2,	fread_number( fp ) ); /* Added by Tarl 02 Mar 02 */
		KEY( "Smount",	deity->smount,	fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
		KEY( "Sminion",	deity->sminion,	fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
	    KEY( "Srecall",	deity->srecall,		fread_number( fp ) );
	    KEY( "Sex",		deity->sex,		fread_number( fp ) );
	    KEY( "Spell_aid",   deity->spell_aid,	fread_number( fp ) );
		KEY( "Spell1",	deity->spell1,		fread_number( fp ) ); /* Added by Tarl 24 Mar 02 */
		KEY( "Spell2",	deity->spell2,		fread_number( fp ) ); /* Added by Tarl 24 Mar 02 */
		KEY( "Spell3",	deity->spell3,		fread_number( fp ) ); /* Added by Tarl 24 Mar 02 */
		KEY( "Sspell1",	deity->sspell1,		fread_number( fp ) ); /* Added by Tarl 24 Mar 02 */
		KEY( "Sspell2",	deity->sspell2,		fread_number( fp ) ); /* Added by Tarl 24 Mar 02 */
		KEY( "Sspell3",	deity->sspell3,		fread_number( fp ) ); /* Added by Tarl 24 Mar 02 */

	    KEY( "Steal",	deity->steal,		fread_number( fp ) );
	    KEY( "Suscept",	deity->suscept,		fread_number( fp ) );
        KEY( "Suscept2",    deity->suscept2,        fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
        KEY( "Suscept3",    deity->suscept3,        fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
	    KEY( "Susceptnum",  deity->susceptnum,	fread_number( fp ) );
        KEY( "Susceptnum2", deity->susceptnum2,     fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
        KEY( "Susceptnum3", deity->susceptnum3,     fread_number( fp ) ); /* Added by Tarl 24 Feb 02 */
	    break;

	case 'W':
	    KEY( "Worshippers",	deity->worshippers,	fread_number( fp ) );
	    break;
	}
	if( !fMatch )
	    bug( "Fread_deity: no match: %s", word );
    }
}

/* Load a deity file */
bool load_deity_file( const char *deityfile )
{
     char filename[256];
     DEITY_DATA *deity;
     FILE *fp;
     bool found;

     found = FALSE;
   snprintf( filename, 256, "%s%s", DEITY_DIR, deityfile );

   if( ( fp = fopen( filename, "r" ) ) != NULL )
     {
	for ( ; ; )
	{
	    char letter;
	    char *word;

	    letter = fread_letter( fp );
	    if ( letter == '*' )
	    {
		fread_to_eol( fp );
		continue;
	    }

	    if ( letter != '#' )
	    {
		bug( "%s", "Load_deity_file: # not found." );
		break;
	    }

	    word = fread_word( fp );
	    if ( !str_cmp( word, "DEITY" ) )
	    {
		CREATE( deity, DEITY_DATA, 1 );
		fread_deity( deity, fp );
		LINK(deity, first_deity, last_deity, next, prev);
		found = TRUE;
		break;
	    }
	    else
	    {
		bug( "Load_deity_file: bad section: %s.", word );
		break;
	    }
	}
	FCLOSE( fp );
    }

    return found;
}

/* Load in all the deity files */
void load_deity( void )
{
    FILE *fpList;
   const char *filename;
    char deitylist[256];

    first_deity = NULL;
    last_deity  = NULL;

    log_string( "Loading deities..." );

   snprintf( deitylist, 256, "%s%s", DEITY_DIR, DEITY_LIST );
   if( !( fpList = fopen ( deitylist, "r" ) ) )
    {
	perror( deitylist );
	exit( 1 );
    }

    for ( ; ; )
    {
	filename = feof( fpList ) ? "$" : fread_word( fpList );
	if ( filename[0] == '$' )
	   break;
	if ( !load_deity_file( filename ) )
	  bug( "Cannot load deity file: %s", filename );
    }	
    FCLOSE( fpList );
    log_string( "Done deities" );
    return;
}

/* Function modified from original form, support added for recallroom + avatar & object restored - Samson */
CMDF do_setdeity( CHAR_DATA *ch, char *argument )
{
   char arg1[MIL], arg2[MIL], arg3[MIL];
    DEITY_DATA *deity;
    int value;

   if( IS_NPC( ch ) )
    {
	send_to_char( "Huh?\n\r", ch );
	return;
    }

    switch( ch->substate )
    {
        default:
		break;
 
	case SUB_RESTRICTED:
		send_to_char( "You cannot do this while in another command.\n\r", ch );
		return;

	case SUB_DEITYDESC:  
	   deity = (DEITY_DATA *)ch->pcdata->dest_buf;
         DISPOSE( deity->deitydesc );
         deity->deitydesc = copy_buffer_nohash( ch );
        	stop_editing( ch );
		save_deity( deity );
      		ch->substate = ch->tempnum;
        	return;

      case SUB_EDIT_ABORT:
         ch->substate = SUB_NONE;
         send_to_char( "Aborting description.\n\r", ch );
         return;
    }

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

   if( arg1[0] == '\0' )
    {
	send_to_char( "Usage: setdeity <deity> <field> <toggle>\n\r", ch );
	send_to_char( "\n\rField being one of:\n\r", ch );
	send_to_char( "filename name description alignment worshippers npcfoe susceptnum\n\r", ch );
	send_to_char( "race race2 npcrace Class minion element sex affected suscept elementnum affectednum\n\r", ch );
	send_to_char( "recallroom avatar mount object delete element2 element3 affected2 affected3 suscept2 suscept3\n\r", ch );
	send_to_char( "affectednum2 affectednum3 elementnum2 elementnum3 susceptnum2 susceptnum3\n\r", ch);
	send_to_char( "class2 class3 class4 race3 race4 object2\n\r", ch);
	send_to_char( "\n\rFavor adjustments:\n\r", ch );
	send_to_char( "flee flee_npcrace kill kill_npcrace kill_magic\n\r", ch );
	send_to_char( "die die_npcrace dig_corpse bury_corpse spell_aid\n\r", ch);
	send_to_char( "steal backstab aid aid_spell sac kill_npcfoe\n\r", ch );
	send_to_char( "die_npcfoe flee_npcfoe\n\r", ch );
	send_to_char( "\n\rFavor requirements for supplicate:\n\r", ch );
	send_to_char( "scorpse savatar smount sdeityobj sdeityobj2 srecall\n\r\n\r", ch );
	send_to_char( "spell1 spell2 spell3 Sspell1 Sspell2 Sspell3\n\r", ch ); /* Added by Tarl 24 Mar 02 */
	send_to_char( "Objstat - being one of:\n\r", ch );
	send_to_char( "str int wis con dex cha lck\n\r", ch );
	send_to_char( " 0 - 1 - 2 - 3 - 4 - 5 - 6\n\r", ch );
	return;
    }

    deity = get_deity( arg1 );
   if( !deity )
    {
 	send_to_char( "No such deity.\n\r", ch );
	return;
    }

   if( !str_cmp( arg2, "delete" ) )
    {
	char filename[256];

	snprintf( filename, 256, "%s%s", DEITY_DIR, deity->filename );
      free_deity( deity );
	ch_printf( ch, "&YDeity information for %s deleted.\n\r", arg1 );

	if( !remove( filename ) )
	   ch_printf( ch, "&RDeity file for %s destroyed.\n\r", arg1 );

      write_deity_list();
	return;
    }

   if( !str_cmp( arg2, "name" ) )
    {
	STRFREE( deity->name );
	deity->name = STRALLOC( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "avatar" ) )
    {
	deity->avatar = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   /* Added by Tarl 24 Feb 02 */
   if( !str_cmp( arg2, "mount" ) )
    {
	deity->mount = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   /* Added by Tarl 24 Feb 02 */
   if( !str_cmp( arg2, "minion" ) )
    {
	deity->minion = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "object" ) )
    {
	deity->deityobj = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "object2" ) )
    {
	deity->deityobj2 = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "recallroom" ) )
    {
	deity->recallroom = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "filename" ) )
    {
	DISPOSE( deity->filename );
	deity->filename = str_dup( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	write_deity_list( );
	return;
    }

   if( !str_cmp( arg2, "description" ) )
    {
	if( ch->substate == SUB_REPEATCMD )
	  ch->tempnum = SUB_REPEATCMD;
	else
          ch->tempnum = SUB_NONE;
        ch->substate = SUB_DEITYDESC;
        ch->pcdata->dest_buf = deity;
      start_editing( ch, deity->deitydesc );
	  editor_desc_printf( ch, "Deity description for deity '%s'.", deity->name );
        return;
    }

   if( !str_cmp( arg2, "alignment" ) )
    {
	deity->alignment = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "flee" ) )
    {
	deity->flee = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "flee_npcrace" ) )
    {
        deity->flee_npcrace = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "flee_npcfoe" ) )
    {
	deity->flee_npcfoe = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "kill" ) )
    {
        deity->kill = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "kill_npcrace" ) )
    {
        deity->kill_npcrace = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "kill_npcfoe" ) )
    {
	deity->kill_npcfoe = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "kill_magic" ) )
    {
        deity->kill_magic = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "sac" ) )
    {
        deity->sac = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "bury_corpse" ) )
    {
        deity->bury_corpse = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "aid_spell" ) )
    {
        deity->aid_spell = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "aid" ) )
    {
        deity->aid = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "steal" ) )
    {
        deity->steal = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "backstab" ) )
    {
        deity->backstab = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "die" ) )
    {
        deity->die = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "die_npcrace" ) )
    {
        deity->die_npcrace = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "die_npcfoe" ) )
    {
	deity->die_npcfoe = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "spell_aid" ) )
    {
        deity->spell_aid = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "dig_corpse" ) )
    {
        deity->dig_corpse = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "scorpse" ) )
    {
        deity->scorpse = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "savatar" ) )
    {
        deity->savatar = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   /* Added by Tarl 24 Feb 02 */
   if( !str_cmp( arg2, "smount" ) )
    {
        deity->smount = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   /* Added by Tarl 24 Feb 02 */
   if( !str_cmp( arg2, "sminion" ) )
    {
        deity->sminion = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "sdeityobj" ) )
    {
        deity->sdeityobj = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   /* Added by Tarl 24 Feb 02 */
   if( !str_cmp( arg2, "sdeityobj2" ) )
    {
        deity->sdeityobj2 = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "objstat" ) )
    {
	deity->objstat = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "srecall" ) )
    {
        deity->srecall = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "worshippers" ) )
    {
	deity->worshippers = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "race" ) )
    {
 	value = get_pc_race( argument );
  	if( value < 0 )
         value = atoi( argument );
 	if( value < 0 || value >= MAX_PC_RACE )
	{
	  deity->race = -1;
	  send_to_char( "No race set.\n\r", ch );
	  return;
	}
	deity->race = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "race2" ) )
    {
        value = get_pc_race( argument );
      if( value < 0 )
         value = atoi( argument );
      if( value < 0 || value >= MAX_PC_RACE )
        {
          deity->race2 = -1;
          send_to_char( "No race set.\n\r", ch );
          return;
        }
        deity->race2 = value;
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   /* Added by Tarl 25 Feb 02 */
   if( !str_cmp( arg2, "race3" ) )
    {
 	value = get_pc_race( argument );
  	if( value < 0 )
         value = atoi( argument );
 	if( value < 0 || value >= MAX_PC_RACE )
	{
	  deity->race3 = -1;
	  send_to_char( "No race set.\n\r", ch );
	  return;
	}
	deity->race3 = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   /* Added by Tarl 25 Feb 02 */
   if( !str_cmp( arg2, "race4" ) )
    {
        value = get_pc_race( argument );
      if( value < 0 )
         value = atoi( argument );
      if( value < 0 || value >= MAX_PC_RACE )
        {
          deity->race4 = -1;
          send_to_char( "No race set.\n\r", ch );
          return;
        }
        deity->race4 = value;
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "npcrace" ) )
    {
	value = get_npc_race( argument );
	if( value < 0 )
         value = atoi( argument );
	if( value < 0 || value >= MAX_NPC_RACE )
	{
	  send_to_char( "Invalid npc race.\n\r", ch );
	  return;
	}	
	deity->npcrace = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "npcfoe" ) )
    {
	value = get_npc_race( argument );
	if( value < 0 )
         value = atoi( argument );
	if( value < 0 || value >= MAX_NPC_RACE )
	{
	  send_to_char( "Invalid npc race.\n\r", ch );
	  return;
	}
	deity->npcfoe = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   /* Modified to allow setting Class by name instead of number - Samson 8-2-98 */
   if( !str_cmp( arg2, "Class" ) )
    {
	value = get_npc_class( argument );
	if( value < 0 )
         value = atoi( argument );
	if( value < 0 || value >= MAX_PC_CLASS )
	{
	   deity->Class = -1;
	   send_to_char( "No Class set.\n\r", ch );
	  return;
	}
	deity->Class = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   /* Added by Tarl 25 Feb 02 */
   if( !str_cmp( arg2, "class2" ) )
    {
	value = get_npc_class( argument );
	if( value < 0 )
         value = atoi( argument );
	if( value < 0 || value >= MAX_PC_CLASS )
	{
	   deity->Class2 = -1;
	   send_to_char( "No Class set.\n\r", ch );
	  return;
	}
	deity->Class2 = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   /* Added by Tarl 25 Feb 02 */
   if( !str_cmp( arg2, "class3" ) )
    {
	value = get_npc_class( argument );
	if( value < 0 )
         value = atoi( argument );
	if( value < 0 || value >= MAX_PC_CLASS )
	{
	   deity->Class3 = -1;
	   send_to_char( "No Class set.\n\r", ch );
	  return;
	}
	deity->Class3 = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   /* Added by Tarl 25 Feb 02 */
   if( !str_cmp( arg2, "class4" ) )
    {
	value = get_npc_class( argument );
	if( value < 0 )
         value = atoi( argument );
	if( value < 0 || value >= MAX_PC_CLASS )
	{
	   deity->Class4 = -1;
	   send_to_char( "No Class set.\n\r", ch );
	  return;
	}
	deity->Class4 = value;
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "susceptnum" ) )
    {
	deity->susceptnum = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "susceptnum2" ) ) /* Added by Tarl 24 Feb 02 */
    {
        deity->susceptnum2 = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "susceptnum3" ) ) /* Added by Tarl 24 Feb 02 */
    {
        deity->susceptnum3 = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }
   if( !str_cmp( arg2, "elementnum" ) )
    {
	deity->elementnum = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "elementnum2" ) ) /* Added by Tarl 24 Feb 02 */
    {
        deity->elementnum2 = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "elementnum3" ) ) /* Added by Tarl 24 Feb 02 */
    {
        deity->elementnum3 = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "affectednum" ) )
    {
	deity->affectednum = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "affectednum2" ) ) /* Added by Tarl 24 Feb 02 */
    {
      deity->affectednum2 = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "affectednum3" ) ) /* Added by Tarl 24 Feb 02 */
    {
      deity->affectednum3 = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "spell1" ) ) /* Added by Tarl 24 Mar 02 */
    {
	if( skill_lookup( argument ) < 0 )
      {
	   send_to_char( "No skill/spell by that name.\n\r", ch );
		return;
	}
	deity->spell1 = skill_lookup( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "spell2" ) ) /* Added by Tarl 24 Mar 02 */
    {
	if( skill_lookup( argument ) < 0 )
      {
	   send_to_char( "No skill/spell by that name.\n\r", ch );
	   return;
	}
	deity->spell2 = skill_lookup( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "spell3" ) ) /* Added by Tarl 24 Mar 02 */
    {
	if( skill_lookup( argument ) < 0 )
      {
	   send_to_char("No skill/spell by that name.\n\r", ch );
	   return;
	}
	deity->spell3 = skill_lookup( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }
   if( !str_cmp( arg2, "Sspell1" ) ) /* Added by Tarl 24 Mar 02 */
    {
	deity->sspell1 = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "Sspell2" ) ) /* Added by Tarl 24 Mar 02 */
    {
	deity->sspell2 = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "Sspell3" ) ) /* Added by Tarl 24 Mar 02 */
    {
	deity->sspell3 = atoi( argument );
        send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "suscept" ) )
    {
      bool fMatch = FALSE;
 
      while( argument[0] != '\0' )
        {
           argument = one_argument( argument, arg3 );
         if( !str_cmp ( arg3, "none" ) )
            {
            fMatch = TRUE;
            deity->suscept = 0;
            }
           else
            {
             value = get_risflag( arg3 );
            if( value < 0 || value > 31 )
              ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
             else
             {
              TOGGLE_BIT( deity->suscept, 1 << value );
              fMatch = TRUE;
             }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "suscept2" ) ) /* Added by Tarl 24 Feb 02 */
    {
      bool fMatch = FALSE;

      while( argument[0] != '\0' )
        {
           argument = one_argument( argument, arg3 );
         if( !str_cmp ( arg3, "none" ) )
            {
            fMatch = TRUE;
            deity->suscept2 = 0;
            }
           else
            {
             value = get_risflag( arg3 );
            if( value < 0 || value > 31 )
              ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
             else
             {
              TOGGLE_BIT( deity->suscept2, 1 << value );
              fMatch = TRUE;
             }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "suscept3" ) ) /* Added by Tarl 24 Feb 02 */
    {
      bool fMatch = FALSE;

      while( argument[0] != '\0' )
        {
           argument = one_argument( argument, arg3 );
         if( !str_cmp ( arg3, "none" ) )
            {
            fMatch = TRUE;
            deity->suscept3 = 0;
            }
           else
            {
             value = get_risflag( arg3 );
            if( value < 0 || value > 31 )
              ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
             else
             {
              TOGGLE_BIT( deity->suscept3, 1 << value );
              fMatch = TRUE;
             }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "element" ) )
    {
      bool fMatch = FALSE;
 
      while( argument[0] != '\0' )
        {
           argument = one_argument( argument, arg3 );
         if( !str_cmp ( arg3, "none" ) )
            {
            fMatch = TRUE;
            deity->element = 0;
            }
           else
            {
             value = get_risflag( arg3 );
            if( value < 0 || value > 31 )
              ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
             else
             {
              TOGGLE_BIT( deity->element, 1 << value );
              fMatch = TRUE;
             }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "element2" ) ) /* Added by Tarl 24 Feb 02 */
    {
      bool fMatch = FALSE;

      while( argument[0] != '\0' )
        {
           argument = one_argument( argument, arg3 );
         if( !str_cmp ( arg3, "none" ) )
            {
            fMatch = TRUE;
            deity->element2 = 0;
            }
           else
            {
             value = get_risflag( arg3 );
            if( value < 0 || value > 31 )
              ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
             else
             {
              TOGGLE_BIT( deity->element2, 1 << value );
              fMatch = TRUE;
             }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "element3" ) ) /* Added by Tarl 24 Feb 02 */
    {
      bool fMatch = FALSE;

      while( argument[0] != '\0' )
        {
           argument = one_argument( argument, arg3 );
         if( !str_cmp( arg3, "none" ) )
            {
            fMatch = TRUE;
            deity->element3 = 0;
            }
           else
            {
             value = get_risflag( arg3 );
            if( value < 0 || value > 31 )
              ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
             else
             {
              TOGGLE_BIT( deity->element3, 1 << value );
              fMatch = TRUE;
             }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }


    if ( !str_cmp( arg2, "sex" ) )
    {
	deity->sex = atoi( argument );
	send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "affected" ) )
    {
	bool fMatch = FALSE;

	while( argument[0] != '\0' )      
	{
	    argument = one_argument( argument, arg3 );
	   if( !str_cmp( arg3, "none" ) )
	    {
		fMatch = TRUE;
		xCLEAR_BITS( deity->affected );
            }
	    else
	    {
		value = get_aflag( arg3 );
		if( value < 0 || value > MAX_BITS )
		    ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
		else
		{
        	    xTOGGLE_BIT( deity->affected, value );
        	    fMatch = TRUE;
		}
	    }
	}
	if( fMatch )
		send_to_char( "Done.\n\r", ch );
	save_deity( deity );
	return;
    }

   if( !str_cmp( arg2, "affected2" ) ) /* Added by Tarl 24 Feb 02 */
    {
        bool fMatch = FALSE;

      while( argument[0] != '\0' )   
        {
            argument = one_argument( argument, arg3 );
         if( !str_cmp ( arg3, "none" ) )
            {
                fMatch = TRUE;
            xCLEAR_BITS( deity->affected2 );
            }
            else
            {
                value = get_aflag( arg3 );
            if( value < 0 || value > MAX_BITS )
                    ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
                else
                {
                    xTOGGLE_BIT( deity->affected2, value );
                    fMatch = TRUE;
                }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

   if( !str_cmp( arg2, "affected3" ) ) /* Added by Tarl 24 Feb 02 */
    {
        bool fMatch = FALSE;

      while( argument[0] != '\0' )   
        {
            argument = one_argument( argument, arg3 );
         if( !str_cmp ( arg3, "none" ) )
            {
                fMatch = TRUE;
            xCLEAR_BITS( deity->affected3 );
            }
            else
            {
                value = get_aflag( arg3 );
            if( value < 0 || value > MAX_BITS )
                    ch_printf( ch, "Unknown flag: %s\n\r", arg3 );
                else
                {
                    xTOGGLE_BIT( deity->affected3, value );
                    fMatch = TRUE;
                }
            }
        }
      if( fMatch )
		send_to_char( "Done.\n\r", ch );
        save_deity( deity );
        return;
    }

    do_setdeity( ch, "" );
    return;
}

/* Modified to show recallroom, and restored to show avatar + object - Samson 8-1-98 */
/* Regrouped by Adjani, 'cause Dwip said so -Adjani, 12-03-2002*/
CMDF do_showdeity( CHAR_DATA *ch, char *argument )
{
    DEITY_DATA *deity;
   if( IS_NPC( ch ) )
    {
	send_to_char( "Huh?\n\r", ch );
	return;
    }
   if( argument[0] == '\0' )
    {
	send_to_char( "Usage: showdeity <deity>\n\r", ch );
	return;
    }
    deity = get_deity( argument );
   if( !deity )
    {
	send_to_char( "No such deity.\n\r", ch );
	return;
    }
    ch_printf( ch, "Deity: %s  Filename: %s\n\rDescription:\n\r%s\n\r",
		deity->name, deity->filename, deity->deitydesc );   
    ch_printf( ch, "Alignment: %-7dNpcrace: %-9sNpcfoe: %-8sSex: %s\n\r\n\r", 
	deity->alignment, 
	( deity->npcrace < 0 || deity->npcrace > MAX_NPC_RACE ) ? "none" : npc_race[deity->npcrace],
		( deity->npcfoe < 0 || deity->npcfoe > MAX_NPC_RACE ) ? "none" : npc_race[deity->npcfoe],
		deity->sex == -1	  ? "none"   : npc_sex[deity->sex] );
    ch_printf( ch, "Race: %-11sRace2: %-11sRace3: %-11sRace4: %s\n\r", 
 	( deity->race < 0 || deity->race > MAX_PC_RACE ) ? "none" : race_table[deity->race]->race_name,
	 	( deity->race2 < 0 || deity->race2 > MAX_PC_RACE ) ? "none" : race_table[deity->race2]->race_name, 
		( deity->race3 < 0 || deity->race3 > MAX_PC_RACE ) ? "none" : race_table[deity->race3]->race_name,
		( deity->race4 < 0 || deity->race4 > MAX_PC_RACE ) ? "none" : race_table[deity->race4]->race_name );;
	ch_printf( ch, "Class: %-10sClass2: %-10sClass3: %-10sClass4: %s\n\r\n\r", 
	 	( deity->Class < 0 || deity->Class > MAX_PC_CLASS ) ? "none" : class_table[deity->Class]->who_name, 
		( deity->Class2 < 0 || deity->Class2 > MAX_PC_CLASS ) ? "none" : class_table[deity->Class2]->who_name,
		( deity->Class3 < 0 || deity->Class3 > MAX_PC_CLASS ) ? "none" : class_table[deity->Class3]->who_name,
		( deity->Class4 < 0 || deity->Class4 > MAX_PC_CLASS ) ? "none" : class_table[deity->Class4]->who_name );
    ch_printf( ch, "Affectednum: %-6dElementnum: %-7dSusceptnum: %-7dSpell1: %s\n\r",
		deity->affectednum, deity->elementnum, deity->susceptnum, skill_table[deity->spell1]->name );
    ch_printf( ch, "Affectednum2: %-5dElementnum2: %-6dSusceptnum2: %-6dSpell2: %s\n\r",
		deity->affectednum2, deity->elementnum2, deity->susceptnum2, skill_table[deity->spell2]->name );
    ch_printf( ch, "Affectednum3: %-5dElementnum3: %-6dSusceptnum3: %-6dSpell3: %s\n\r",
		deity->affectednum3, deity->elementnum3, deity->susceptnum3, skill_table[deity->spell3]->name );
    ch_printf( ch, "\n\rAffected: %-12s  ", affect_bit_name(&deity->affected) );
    ch_printf( ch, "Affected2: %-10s   ", affect_bit_name(&deity->affected2) );
    ch_printf( ch, "Affected3: %s\n\r", affect_bit_name(&deity->affected3) );
    ch_printf( ch, "Suscept: %-11s    ", flag_string( deity->suscept, ris_flags ));
    ch_printf( ch, "Suscept2: %-10s    ", flag_string( deity->suscept2, ris_flags ));
    ch_printf( ch, "Suscept3: %s\n\r", flag_string( deity->suscept3, ris_flags ));
    ch_printf( ch, "Element: %-12s   ", flag_string( deity->element, ris_flags ));
    ch_printf( ch, "Element2: %-11s   ", flag_string( deity->element2, ris_flags ));
    ch_printf( ch, "Element3: %s\n\r", flag_string( deity->element3, ris_flags ));
    ch_printf( ch, "Sspell1: %-15dSspell2: %-15dSspell3: %d\n\r",
            deity->sspell1, deity->sspell2, deity->sspell3 );
    ch_printf( ch, "\n\rFlee: %-12dFlee_npcrace: %-4dKill_npcrace: %-4dKill: %d\n\r",
		deity->flee, deity->flee_npcrace,	deity->kill_npcrace, deity->kill );
    ch_printf( ch, "Kill_magic: %-6dSac: %-13dBury_corpse: %-5dAid_spell: %d\n\r",
		deity->kill_magic, deity->sac, deity->bury_corpse, deity->aid_spell );
    ch_printf( ch, "Aid: %-13dSteal: %-11dBackstab: %-8dDie: %d\n\r", 
	deity->aid, deity->steal, deity->backstab, deity->die );
    ch_printf( ch, "Die_npcrace: %-5dDig_corpse: %-6dSpell_aid: %-7dKill_npcfoe: %d\n\r",
	deity->die_npcrace, deity->dig_corpse, deity->spell_aid, deity->kill_npcfoe );
    ch_printf( ch, "Die_npcfoe: %-6dFlee_npcfoe: %d\n\r\n\r",
		deity->die_npcfoe, deity->flee_npcfoe );
    ch_printf( ch, "Object: %-9d Object2: %-8d Avatar: %-10d Mount: %d\n\r",
		deity->deityobj, deity->deityobj2, deity->avatar, deity->mount );
	ch_printf( ch, "Minion: %-10dScorpse: %-9dSavatar: %-10dSmount: %-9d\n\r",
		deity->minion, deity->scorpse, deity->savatar, deity->smount );
	ch_printf( ch, "Sminion: %-9dSdeityobj: %-7dSdeityobj2: %-7dSrecall: %-9d\n\r",
		deity->sminion, deity->sdeityobj, deity->sdeityobj, deity->srecall );
    ch_printf( ch, "Recallroom: %-6dObjstat: %-9dWorshippers: %d\n\r", 
                deity->recallroom, deity->objstat, deity->worshippers );
    return;
}

CMDF do_makedeity( CHAR_DATA *ch, char *argument )
{
    char filename[256];
    DEITY_DATA *deity;
    bool found;

   if( !argument || argument[0] == '\0' )
    {
	send_to_char( "Usage: makedeity <deity name>\n\r", ch );
	return;
    }

    found = FALSE;
   snprintf( filename, 256, "%s%s", DEITY_DIR, strlower(argument) );
    CREATE( deity, DEITY_DATA, 1 );
    LINK( deity, first_deity, last_deity, next, prev );
    deity->name		= STRALLOC( argument );
    deity->filename	= str_dup( filename );
    write_deity_list();
    save_deity( deity );
}

CMDF do_devote( CHAR_DATA *ch, char *argument )
{
    DEITY_DATA *deity;

   if( IS_NPC( ch ) )
    {
	send_to_char( "Huh?\n\r", ch );
	return;
    }

   if( !argument || argument[0] == '\0' )
    {
	send_to_char( "Devote yourself to which deity?\n\r", ch );
	return;
    }

   if ( !str_cmp( argument, "none" ) )
    {
        AFFECT_DATA af;
      if( !ch->pcdata->deity )
        {
           send_to_char( "You have already chosen to worship no deities.\n\r", ch );
           return;
        }
        --ch->pcdata->deity->worshippers;
        ch->pcdata->favor = -2500;
	ch->mental_state = -80;
        send_to_char( "A terrible curse afflicts you as you forsake a deity!\n\r", ch );
	xREMOVE_BITS( ch->affected_by, ch->pcdata->deity->affected );
        xREMOVE_BITS( ch->affected_by, ch->pcdata->deity->affected2 );
        xREMOVE_BITS( ch->affected_by, ch->pcdata->deity->affected3 );
	REMOVE_RESIS( ch, ch->pcdata->deity->element );
      REMOVE_RESIS( ch, ch->pcdata->deity->element2 );
      REMOVE_RESIS( ch, ch->pcdata->deity->element3 );
	REMOVE_SUSCEP( ch, ch->pcdata->deity->suscept );
      REMOVE_SUSCEP( ch, ch->pcdata->deity->suscept2 );
      REMOVE_SUSCEP( ch, ch->pcdata->deity->suscept3 );
	affect_strip( ch, gsn_blindness );
        af.type         = gsn_blindness;
        af.location     = APPLY_HITROLL;
        af.modifier     = -4;
      af.duration     = (int)( 50 * DUR_CONV );
        af.bitvector    = meb(AFF_BLIND);
        affect_to_char( ch, &af );
      save_deity( ch->pcdata->deity );
        send_to_char( "You cease to worship any deity.\n\r", ch );
        ch->pcdata->deity = NULL;
      STRFREE( ch->pcdata->deity_name );
        save_char_obj( ch );
        return;
    }

   deity = get_deity( argument );
   if( !deity )
    {
	send_to_char( "No such deity holds weight on this world.\n\r", ch );
	return;
    }

   if( ch->pcdata->deity )
    {
	send_to_char( "You are already devoted to a deity.\n\r", ch );
	return;
    }

   /* Edited by Tarl 25 Feb 02 */
   if( ( deity->Class != -1 ) && ( ( deity->Class != ch->Class ) && ( deity->Class2 != ch->Class ) 
      && ( deity->Class3 != ch->Class ) && ( deity->Class4 != ch->Class ) ) )
    {
	send_to_char( "That deity will not accept your worship due to your class.\n\r", ch );
	return;
    }

   if( ( deity->sex != -1 ) && ( deity->sex != ch->sex ) )
    {
	send_to_char( "That deity will not accept worshippers of your sex.\n\r", ch );
	return;
    }

	/* Edited by Tarl 25 Feb 02 */
   if( ( deity->race != -1 ) && ( ( deity->race != ch->race ) && ( deity->race2 != ch->race )
      && ( deity->race3 != ch->race ) && ( deity->race4 != ch->race ) ) )
    {
	send_to_char( "That deity will not accept worshippers of your race.\n\r", ch );
	return;
    }
    
   STRFREE( ch->pcdata->deity_name );
    ch->pcdata->deity_name = QUICKLINK( deity->name );
    ch->pcdata->deity = deity;
    
    act( AT_MAGIC, "Body and soul, you devote yourself to $t!", ch, ch->pcdata->deity_name, NULL, TO_CHAR );
    ++ch->pcdata->deity->worshippers;
   save_deity( ch->pcdata->deity );
    save_char_obj( ch );
    return;
}

CMDF do_deities( CHAR_DATA *ch, char *argument )
{
    DEITY_DATA *deity;
    int count = 0;

   if ( !argument || argument[0] == '\0' )
    {
	send_to_pager( "&gFor detailed information on a deity, try 'deities <deity>' or 'help deities'\n\r", ch );
	send_to_pager( "Deity			Worshippers\n\r", ch );
	for ( deity = first_deity; deity; deity = deity->next )
	{
	   pager_printf( ch, "&G%-14s	&g%19d\n\r", deity->name, deity->worshippers );
	   count++;
	}
	if ( !count )
	{
	   send_to_pager( "&gThere are no deities on this world.\n\r", ch );
	    return;
	}
        return;
     }

    deity = get_deity( argument );
    if ( !deity )
    {
	send_to_pager( "&gThat deity does not exist.\n\r", ch );
	return;
    }

   pager_printf( ch, "&gDeity:        &G%s\n\r", deity->name );
   pager_printf( ch, "&gDescription:\n\r&G%s", deity->deitydesc );
    return;
}

/*
Internal function to adjust favor.
Fields are:
0 = flee		5 = sac			10 = backstab	
1 = flee_npcrace	6 = bury_corpse		11 = die
2 = kill		7 = aid_spell		12 = die_npcrace
3 = kill_npcrace	8 = aid			13 = spell_aid
4 = kill_magic		9 = steal		14 = dig_corpse
15 = die_npcfoe	       16 = flee_npcfoe         17 = kill_npcfoe
*/
void adjust_favor( CHAR_DATA *ch, int field, int mod )
{
    int oldfavor;
    
   if( IS_NPC( ch ) || !ch->pcdata->deity )
	return;

    oldfavor = ch->pcdata->favor;

   if( ( ch->alignment - ch->pcdata->deity->alignment > 650
    ||   ch->alignment - ch->pcdata->deity->alignment < -650 ) && ch->pcdata->deity->alignment != 0 )
    {
	ch->pcdata->favor -= 2;
	ch->pcdata->favor = URANGE( -2500, ch->pcdata->favor, 5000 );

	if( ch->pcdata->favor > ch->pcdata->deity->affectednum )
    {
	   if( !xIS_EMPTY( ch->pcdata->deity->affected ) && !xHAS_BITS( ch->affected_by, ch->pcdata->deity->affected ) )
	{
		ch_printf( ch, "%s looks favorably upon you and bestows %s as a reward.\n\r",
		   ch->pcdata->deity_name, ext_flag_string( &ch->pcdata->deity->affected, a_flags ) );
		xSET_BITS( ch->affected_by, ch->pcdata->deity->affected );
	}
	}
	if( ch->pcdata->favor > ch->pcdata->deity->affectednum2 )
	{
	   if( !xIS_EMPTY( ch->pcdata->deity->affected2 ) && !xHAS_BITS( ch->affected_by, ch->pcdata->deity->affected2 ) )
	   {
		ch_printf( ch, "%s looks favorably upon you and bestows %s as a reward.\n\r",
		   ch->pcdata->deity_name, ext_flag_string( &ch->pcdata->deity->affected2, a_flags ) );
		xSET_BITS( ch->affected_by, ch->pcdata->deity->affected2 );
	}
	}
	if( ch->pcdata->favor > ch->pcdata->deity->affectednum3 )
	{
	   if( !xIS_EMPTY( ch->pcdata->deity->affected3 ) && !xHAS_BITS( ch->affected_by, ch->pcdata->deity->affected3 ) )
	   {
		ch_printf( ch, "%s looks favorably upon you and bestows %s as a reward.\n\r",
		   ch->pcdata->deity_name, ext_flag_string( &ch->pcdata->deity->affected3, a_flags ) );
		xSET_BITS( ch->affected_by, ch->pcdata->deity->affected3 );
	   }		
	}
      if( ch->pcdata->favor > ch->pcdata->deity->elementnum )
		{
	   if( ch->pcdata->deity->element != 0 && !IS_RESIS( ch, ch->pcdata->deity->element ) )
	   {
		SET_RESIS( ch, ch->pcdata->deity->element );
		ch_printf( ch, "%s looks favorably upon you and bestows %s resistance upon you.\n\r",
		   ch->pcdata->deity_name, flag_string( ch->pcdata->deity->element, ris_flags ) );
		}
      }
      if( ch->pcdata->favor > ch->pcdata->deity->elementnum2 )
      {
	   if( ch->pcdata->deity->element2 != 0 && !IS_RESIS( ch, ch->pcdata->deity->element2 ) )
	   {
		SET_RESIS( ch, ch->pcdata->deity->element2 );
		ch_printf( ch, "%s looks favorably upon you and bestows %s resistance upon you.\n\r",
		   ch->pcdata->deity_name, flag_string( ch->pcdata->deity->element2, ris_flags ) );
	   }
	}
      if( ch->pcdata->favor > ch->pcdata->deity->elementnum3 )
	{
	   if( ch->pcdata->deity->element3 != 0 && !IS_RESIS( ch, ch->pcdata->deity->element3 ) )
	   {
		SET_RESIS( ch, ch->pcdata->deity->element3 );
		ch_printf( ch, "%s looks favorably upon you and bestows %s resistance upon you.\n\r",
		   ch->pcdata->deity_name, flag_string( ch->pcdata->deity->element3, ris_flags ) );
	   }
      }
      if( ch->pcdata->favor < ch->pcdata->deity->susceptnum )
      {
	   if( ch->pcdata->deity->suscept != 0 && !IS_SUSCEP( ch, ch->pcdata->deity->suscept ) )
	   {
		SET_SUSCEP( ch, ch->pcdata->deity->suscept );
		ch_printf( ch, "%s looks poorly upon you and makes you more vulnerable to %s as punishment.\n\r",
		   ch->pcdata->deity_name, flag_string( ch->pcdata->deity->suscept, ris_flags ) );
	   }
      }
      if( ch->pcdata->favor < ch->pcdata->deity->susceptnum2 )
      {
	   if( ch->pcdata->deity->suscept2 != 0 && !IS_SUSCEP( ch, ch->pcdata->deity->suscept2 ) )
	   {
        	SET_SUSCEP( ch, ch->pcdata->deity->suscept2 );
		ch_printf( ch, "%s looks poorly upon you and makes you more vulnerable to %s as punishment.\n\r",
		   ch->pcdata->deity_name, flag_string( ch->pcdata->deity->suscept2, ris_flags ) );
	   }
      }
      if( ch->pcdata->favor < ch->pcdata->deity->susceptnum3 )
      {
	   if( ch->pcdata->deity->suscept3 != 0 && !IS_SUSCEP( ch, ch->pcdata->deity->suscept3 ) )
	   {
        	SET_SUSCEP( ch, ch->pcdata->deity->suscept3 );
		ch_printf( ch, "%s looks poorly upon you and makes you more vulnerable to %s as punishment.\n\r",
		   ch->pcdata->deity_name, flag_string( ch->pcdata->deity->suscept3, ris_flags ) );
	   }
	}

      if( ( oldfavor > ch->pcdata->deity->affectednum  && ch->pcdata->favor <= ch->pcdata->deity->affectednum )
        || (oldfavor > ch->pcdata->deity->affectednum2 && ch->pcdata->favor <= ch->pcdata->deity->affectednum2 )
        || (oldfavor > ch->pcdata->deity->affectednum3 && ch->pcdata->favor <= ch->pcdata->deity->affectednum3 )
        || (oldfavor > ch->pcdata->deity->elementnum   && ch->pcdata->favor <= ch->pcdata->deity->elementnum )
        || (oldfavor > ch->pcdata->deity->elementnum2  && ch->pcdata->favor <= ch->pcdata->deity->elementnum2 )
        || (oldfavor > ch->pcdata->deity->elementnum3  && ch->pcdata->favor <= ch->pcdata->deity->elementnum3 )
        || (oldfavor < ch->pcdata->deity->susceptnum   && ch->pcdata->favor >= ch->pcdata->deity->susceptnum )
        || (oldfavor < ch->pcdata->deity->susceptnum2  && ch->pcdata->favor >= ch->pcdata->deity->susceptnum2 )
        || (oldfavor < ch->pcdata->deity->susceptnum3  && ch->pcdata->favor >= ch->pcdata->deity->susceptnum3 ) )
	{
   	   update_aris( ch );
	}
	
      /* Added by Tarl 24 Mar 02 */
	if( ( oldfavor > ch->pcdata->deity->sspell1 ) && ( ch->pcdata->favor <= ch->pcdata->deity->sspell1 ) )
      {
	   if( ch->level < skill_table[ch->pcdata->deity->spell1]->skill_level[ch->Class]
               &&   ch->level < skill_table[ch->pcdata->deity->spell1]->race_level[ch->race] ) 
		{
			ch->pcdata->learned[ch->pcdata->deity->spell1] = 0;
		}
	}

	if( ( oldfavor > ch->pcdata->deity->sspell2 ) && ( ch->pcdata->favor <= ch->pcdata->deity->sspell2 ) )
	{
	   if( ch->level < skill_table[ch->pcdata->deity->spell2]->skill_level[ch->Class]
            && ch->level < skill_table[ch->pcdata->deity->spell2]->race_level[ch->race] ) 
	   {
			ch->pcdata->learned[ch->pcdata->deity->spell2] = 0;
		}
	}

	if( ( oldfavor > ch->pcdata->deity->sspell3 ) && ( ch->pcdata->favor <= ch->pcdata->deity->sspell3 ) )
	{
	   if( ch->level < skill_table[ch->pcdata->deity->spell3]->skill_level[ch->Class]
           && ch->level < skill_table[ch->pcdata->deity->spell3]->race_level[ch->race] ) 
	   {
			ch->pcdata->learned[ch->pcdata->deity->spell3] = 0;
		}
	}
	return;
   }

   if( mod < 1 )
      mod = 1;
   switch( field )
        {
      default:
      break;
 
	case 0:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->flee / mod);	break;
	case 1:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->flee_npcrace / mod);	 break;
	case 2:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->kill / mod);	break;
	case 3:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->kill_npcrace / mod);	 break;
	case 4:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->kill_magic / mod);	 break;
	case 5:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->sac / mod);	break;
	case 6:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->bury_corpse / mod);	 break;
	case 7:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->aid_spell / mod);	 break;
	case 8:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->aid / mod);	break;	
	case 9:  ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->steal / mod);	break;
	case 10: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->backstab / mod); break;
	case 11: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->die / mod);	break;
	case 12: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->die_npcrace / mod);	 break;
	case 13: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->spell_aid / mod);	 break;
	case 14: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->dig_corpse / mod);	 break;
	case 15: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->die_npcfoe / mod);	 break;
	case 16: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->flee_npcfoe / mod);	 break;
	case 17: ch->pcdata->favor += number_fuzzy(ch->pcdata->deity->kill_npcfoe / mod);	 break;
   }
   ch->pcdata->favor = URANGE( -2500, ch->pcdata->favor, 5000 );

   if( ch->pcdata->favor > ch->pcdata->deity->affectednum )
	  {
	if( !xIS_EMPTY( ch->pcdata->deity->affected ) && !xHAS_BITS( ch->affected_by, ch->pcdata->deity->affected ) )
	{
	   ch_printf( ch, "%s looks favorably upon you and bestows %s as a reward.\n\r",
	      ch->pcdata->deity_name, ext_flag_string( &ch->pcdata->deity->affected, a_flags ) );
	   xSET_BITS( ch->affected_by, ch->pcdata->deity->affected );
	}
   }
   if( ch->pcdata->favor > ch->pcdata->deity->affectednum2 )
   {
      if( !xIS_EMPTY( ch->pcdata->deity->affected2 ) && !xHAS_BITS( ch->affected_by, ch->pcdata->deity->affected2 ) )
	{
	   ch_printf( ch, "%s looks favorably upon you and bestows %s as a reward.\n\r",
		ch->pcdata->deity_name, ext_flag_string( &ch->pcdata->deity->affected2, a_flags ) );
	   xSET_BITS( ch->affected_by, ch->pcdata->deity->affected2 );
	}		
   }
   if( ch->pcdata->favor > ch->pcdata->deity->affectednum3 )
   {
	if( !xIS_EMPTY( ch->pcdata->deity->affected3 ) && !xHAS_BITS( ch->affected_by, ch->pcdata->deity->affected3 ) )
	{
	   ch_printf( ch, "%s looks favorably upon you and bestows %s as a reward.\n\r",
	      ch->pcdata->deity_name, ext_flag_string( &ch->pcdata->deity->affected3, a_flags ) );
	   xSET_BITS( ch->affected_by, ch->pcdata->deity->affected3 );
	}		
   }
   if( ch->pcdata->favor > ch->pcdata->deity->elementnum )
   {
	if( ch->pcdata->deity->element != 0 && !IS_RESIS( ch, ch->pcdata->deity->element ) )
	{
	   SET_RESIS( ch, ch->pcdata->deity->element );
	   ch_printf( ch, "%s looks favorably upon you and bestows %s resistance upon you.\n\r",
	      ch->pcdata->deity_name, flag_string( ch->pcdata->deity->element, ris_flags ) );
	}
   }
   if( ch->pcdata->favor > ch->pcdata->deity->elementnum2 )
   {
	if( ch->pcdata->deity->element2 != 0 && !IS_RESIS( ch, ch->pcdata->deity->element2 ) )
	{
	   SET_RESIS( ch, ch->pcdata->deity->element2 );
	   ch_printf( ch, "%s looks favorably upon you and bestows %s resistance upon you.\n\r",
		ch->pcdata->deity_name, flag_string( ch->pcdata->deity->element2, ris_flags ) );
	}
   }
   if( ch->pcdata->favor > ch->pcdata->deity->elementnum3 )
   {
	if( ch->pcdata->deity->element3 != 0 && !IS_RESIS( ch, ch->pcdata->deity->element3 ) )
	{
	   SET_RESIS( ch, ch->pcdata->deity->element3 );
	   ch_printf( ch, "%s looks favorably upon you and bestows %s resistance upon you.\n\r",
	      ch->pcdata->deity_name, flag_string( ch->pcdata->deity->element3, ris_flags ) );
	}
   }
   if( ch->pcdata->favor < ch->pcdata->deity->susceptnum )
   {
	if( ch->pcdata->deity->suscept != 0 && !IS_SUSCEP( ch, ch->pcdata->deity->suscept ) )
	{
	   SET_SUSCEP( ch, ch->pcdata->deity->suscept );
	   ch_printf( ch, "%s looks poorly upon you and makes you more vulnerable to %s as punishment.\n\r",
		ch->pcdata->deity_name, flag_string( ch->pcdata->deity->suscept, ris_flags ) );
	}
   }
   if( ch->pcdata->favor < ch->pcdata->deity->susceptnum2 )
   {
	if( ch->pcdata->deity->suscept2 != 0 && !IS_SUSCEP( ch, ch->pcdata->deity->suscept2 ) )
	{
         SET_SUSCEP( ch, ch->pcdata->deity->suscept2 );
	   ch_printf( ch, "%s looks poorly upon you and makes you more vulnerable to %s as punishment.\n\r",
	      ch->pcdata->deity_name, flag_string( ch->pcdata->deity->suscept2, ris_flags ) );
	}
   }
   if( ch->pcdata->favor < ch->pcdata->deity->susceptnum3 )
   {
	if( ch->pcdata->deity->suscept3 != 0 && !IS_SUSCEP( ch, ch->pcdata->deity->suscept3 ) )
	{
         SET_SUSCEP( ch, ch->pcdata->deity->suscept3 );
	   ch_printf( ch, "%s looks poorly upon you and makes you more vulnerable to %s as punishment.\n\r",
		ch->pcdata->deity_name, flag_string( ch->pcdata->deity->suscept3, ris_flags ) );
	}
   }
   /* Added by Tarl 24 Mar 02 */
   if( ch->pcdata->favor > ch->pcdata->deity->sspell1 )
   {
      if( ch->pcdata->deity->spell1 != 0 )
      {
         if( ch->pcdata->learned[ch->pcdata->deity->spell1] != 0 )
	     return;
         ch->pcdata->learned[ch->pcdata->deity->spell1] = 95;
      }
	  }

   if( ch->pcdata->favor > ch->pcdata->deity->sspell2 )
	  {
      if( ch->pcdata->deity->spell2 != 0 )
      {
         if( ch->pcdata->learned[ch->pcdata->deity->spell2] != 0 )
	     return;
         ch->pcdata->learned[ch->pcdata->deity->spell2] = 95;
	}
	  }

   if( ch->pcdata->favor > ch->pcdata->deity->sspell3 )
      {
      if( ch->pcdata->deity->spell3 != 0 )
      {
         if( ch->pcdata->learned[ch->pcdata->deity->spell3] != 0 )
            return;
         ch->pcdata->learned[ch->pcdata->deity->spell3] = 95;
	}
	}

   /* If favor crosses over the line then strip the affect */
   if( ( oldfavor > ch->pcdata->deity->affectednum  && ch->pcdata->favor <= ch->pcdata->deity->affectednum )
    || ( oldfavor > ch->pcdata->deity->affectednum2 && ch->pcdata->favor <= ch->pcdata->deity->affectednum2 )
    || ( oldfavor > ch->pcdata->deity->affectednum3 && ch->pcdata->favor <= ch->pcdata->deity->affectednum3 )
    || ( oldfavor > ch->pcdata->deity->elementnum   && ch->pcdata->favor <= ch->pcdata->deity->elementnum )
    || ( oldfavor > ch->pcdata->deity->elementnum2  && ch->pcdata->favor <= ch->pcdata->deity->elementnum2 )
    || ( oldfavor > ch->pcdata->deity->elementnum3  && ch->pcdata->favor <= ch->pcdata->deity->elementnum3 )
    || ( oldfavor < ch->pcdata->deity->susceptnum   && ch->pcdata->favor >= ch->pcdata->deity->susceptnum )
    || ( oldfavor < ch->pcdata->deity->susceptnum2  && ch->pcdata->favor >= ch->pcdata->deity->susceptnum2 )
    || ( oldfavor < ch->pcdata->deity->susceptnum3  && ch->pcdata->favor >= ch->pcdata->deity->susceptnum3 ) )
        {
      update_aris( ch );
        }

        /* Added by Tarl 24 Mar 02 */
   if( ( oldfavor > ch->pcdata->deity->sspell1 ) && ( ch->pcdata->favor <= ch->pcdata->deity->sspell1 ) )
      {
      if( ch->level < skill_table[ch->pcdata->deity->spell1]->skill_level[ch->Class]
            && ch->level < skill_table[ch->pcdata->deity->spell1]->race_level[ch->race] ) 
         {
			ch->pcdata->learned[ch->pcdata->deity->spell1] = 0;
		}
	}

   if( ( oldfavor > ch->pcdata->deity->sspell2 ) && ( ch->pcdata->favor <= ch->pcdata->deity->sspell2 ) )
      {
      if( ch->level < skill_table[ch->pcdata->deity->spell2]->skill_level[ch->Class]
            && ch->level < skill_table[ch->pcdata->deity->spell2]->race_level[ch->race] ) 
         {
			ch->pcdata->learned[ch->pcdata->deity->spell2] = 0;
		}
	}

   if( ( oldfavor > ch->pcdata->deity->sspell3 ) && ( ch->pcdata->favor <= ch->pcdata->deity->sspell3 ) )
      {
      if( ch->level < skill_table[ch->pcdata->deity->spell3]->skill_level[ch->Class]
            && ch->level < skill_table[ch->pcdata->deity->spell3]->race_level[ch->race] ) 
         {
			ch->pcdata->learned[ch->pcdata->deity->spell3] = 0;
		}
	}
        return;
}

/* Modified to support new contintental/planar recall points - Samson 4-13-98 */
CMDF do_supplicate( CHAR_DATA *ch, char *argument )
{
   int oldfavor;

   if( IS_NPC(ch) || !ch->pcdata->deity )
   {
	send_to_char( "You have no deity to supplicate to.\n\r", ch );
	return;
   }

   oldfavor = ch->pcdata->favor;

   if( !argument || argument[0] == '\0' )
   {
	send_to_char( "Supplicate for what?\n\r", ch );
	return;
   }

   if( !str_cmp( argument, "corpse" ) )
   {
	char buf2[MSL], buf3[MSL];
	OBJ_DATA *obj;
	bool found;

	if( ch->pcdata->favor < ch->pcdata->deity->scorpse )
	{
	   send_to_char( "You are not favored enough for a corpse retrieval.\n\r", ch );
	   return;
	}

	if( IS_ROOM_FLAG( ch->in_room, ROOM_CLANSTOREROOM ) )
	{
	   send_to_char( "You cannot supplicate in a storage room.\n\r", ch );
	   return;
	}

	found = FALSE;
	mudstrlcpy( buf3, " ", MSL );
	snprintf( buf2, MSL, "the corpse of %s", ch->name );
	for( obj = first_object; obj; obj = obj->next )
	{
	   if( obj->in_room && !str_cmp( buf2, obj->short_descr ) && ( obj->pIndexData->vnum == OBJ_VNUM_CORPSE_PC ) )
	   {
		found = TRUE;
		if( IS_ROOM_FLAG( obj->in_room, ROOM_NOSUPPLICATE ) )
		{
		   act( AT_MAGIC, "The image of your corpse appears, but suddenly wavers away.", ch, NULL, NULL, TO_CHAR );
		   return;
		}
		act( AT_MAGIC, "Your corpse appears suddenly, surrounded by a divine presence...", ch, NULL, NULL, TO_CHAR );		
		act( AT_MAGIC, "$n's corpse appears suddenly, surrounded by a divine force...", ch, NULL, NULL, TO_ROOM );
		obj_from_room( obj );
		obj = obj_to_room( obj, ch->in_room, ch );
	   }
	}
	if( !found )
	{
	   send_to_char( "No corpse of yours litters the world...\n\r", ch );
	   return;
	}
	ch->pcdata->favor -= ch->pcdata->deity->scorpse;
      adjust_favor( ch, -1, 1 );
	return;
   } 

   if( !str_cmp( argument, "avatar" ) )
   {
	MOB_INDEX_DATA *pMobIndex;
	CHAR_DATA *victim;

      if( ch->pcdata->favor < ch->pcdata->deity->savatar )
        {
           send_to_char( "You are not favored enough for that.\n\r", ch );
           return;
        }
 
      pMobIndex = get_mob_index( ch->pcdata->deity->avatar );
	if( pMobIndex == NULL )
	  {
	   send_to_char( "Your deity has no avatar to pray for.\n\r", ch );
	     return;
	  }

      if( check_pets( ch, pMobIndex ) )
	  {
	   send_to_char( "You have already received an avatar of your deity!\n\r", ch );
	     return;
	  }

        victim = create_mobile( pMobIndex );
        if( !char_to_room( victim, ch->in_room ) )
		log_printf( "char_to_room: %s:%s, line %d.", __FILE__, __FUNCTION__, __LINE__ );
	  fix_maps( ch, victim );

      act( AT_MAGIC, "$n summons a powerful avatar!", ch, NULL, NULL, TO_ROOM );
      act( AT_MAGIC, "You summon a powerful avatar!", ch, NULL, NULL, TO_CHAR );
	bind_follower( victim, ch, gsn_charm_person, (ch->pcdata->favor / 4) + 1 );
	victim->level = LEVEL_AVATAR / 2;
	victim->hit = ch->hit + ch->pcdata->favor;
	victim->alignment = ch->pcdata->deity->alignment;
	victim->max_hit = ch->hit + ch->pcdata->favor;
        
	ch->pcdata->favor -= ch->pcdata->deity->savatar;
      adjust_favor( ch, -1, 1 );  
      return;
        }

   /* Added by Tarl 24 Feb 02 */
   if( !str_cmp( argument, "mount" ) )
      {
	MOB_INDEX_DATA *pMobIndex;
	CHAR_DATA *victim;

      if( ch->pcdata->favor < ch->pcdata->deity->smount )
         {
         send_to_char( "You are not favored enough for that.\n\r", ch );
         return;
	}
 
      pMobIndex = get_mob_index( ch->pcdata->deity->mount );
	if( pMobIndex == NULL )
         {
	   send_to_char( "Your deity has no mount to pray for.\n\r", ch );
	   return;
	}

      if( check_pets( ch, pMobIndex ) )
         {
	   send_to_char( "You have already received a mount of your deity!\n\r", ch );
	   return;
	}

      victim = create_mobile( pMobIndex );
      if( !char_to_room( victim, ch->in_room ) )
	   log_printf( "char_to_room: %s:%s, line %d.", __FILE__, __FUNCTION__, __LINE__ );
	fix_maps( ch, victim );

      act( AT_MAGIC, "$n summons a mount!", ch, NULL, NULL, TO_ROOM );
      act( AT_MAGIC, "You summon a mount!", ch, NULL, NULL, TO_CHAR );
	bind_follower( victim, ch, gsn_charm_person, (ch->pcdata->favor / 4) + 1 );
	ch->pcdata->favor -= ch->pcdata->deity->smount;
      adjust_favor( ch, -1, 1 );
        return;
   } 

   /* Added by Tarl 24 Feb 02 */
   if( !str_cmp( argument, "minion" ) )
   {
	MOB_INDEX_DATA *pMobIndex;
	CHAR_DATA *victim;

      if( ch->pcdata->favor < ch->pcdata->deity->sminion )
        {
           send_to_char( "You are not favored enough for that.\n\r", ch );
           return;
        }
 
        pMobIndex = get_mob_index( ch->pcdata->deity->minion );
	if( pMobIndex == NULL )
	  {
	     send_to_char( "Your deity has no minion to pray for.\n\r", ch );
	     return;
	  }

      if( check_pets( ch, pMobIndex ) )
	  {
	     send_to_char( "You have already received a minion of your deity!\n\r", ch );
	     return;
	  }

        victim = create_mobile( pMobIndex );
        if( !char_to_room( victim, ch->in_room ) )
		log_printf( "char_to_room: %s:%s, line %d.", __FILE__, __FUNCTION__, __LINE__ );
	  fix_maps( ch, victim );

        act( AT_MAGIC, "$n summons a minion!", ch, NULL, NULL, TO_ROOM );
        act( AT_MAGIC, "You summon a minion!", ch, NULL, NULL, TO_CHAR );
	bind_follower( victim, ch, gsn_charm_person, (ch->pcdata->favor / 4) + 1 );
	victim->level = LEVEL_AVATAR / 3;
	victim->hit = (ch->hit / 2) + (ch->pcdata->favor / 4);
	victim->alignment = ch->pcdata->deity->alignment;
	victim->max_hit = (ch->hit / 2) + (ch->pcdata->favor / 4);

	ch->pcdata->favor -= ch->pcdata->deity->sminion;
      adjust_favor( ch, -1, 1 );
        return;
   } 

   if( !str_cmp( argument, "object" ) )
   {
	OBJ_DATA *obj;
	OBJ_INDEX_DATA *pObjIndex;
	AFFECT_DATA *paf;

	if( ch->pcdata->favor < ch->pcdata->deity->sdeityobj )
	{
	   send_to_char( "You are not favored enough for that.\n\r", ch );
	   return;
	}

	pObjIndex = get_obj_index( ch->pcdata->deity->deityobj );
	if ( pObjIndex == NULL )
	{
	   send_to_char( "Your deity has no object to pray for.\n\r", ch );
	   return;
	} 
		
	if( !( obj = create_object( pObjIndex, ch->level ) ) )
      {
         log_printf( "create_object: %s:%s, line %d.", __FILE__, __FUNCTION__, __LINE__ );
         return;
      }
	if( CAN_WEAR( obj, ITEM_TAKE ) )
	   obj = obj_to_char( obj, ch );
	else
	   obj = obj_to_room( obj, ch->in_room, ch );

	act( AT_MAGIC, "$n weaves $p from divine matter!", ch, obj, NULL, TO_ROOM );
	act( AT_MAGIC, "You weave $p from divine matter!", ch, obj, NULL, TO_CHAR );
  	ch->pcdata->favor -= ch->pcdata->deity->sdeityobj;
      adjust_favor( ch, -1, 1 );

        CREATE( paf, AFFECT_DATA, 1 );
        paf->type           = -1;
        paf->duration       = -1;
	switch( ch->pcdata->deity->objstat )
	{
	case 0: paf->location = APPLY_STR; break;
	case 1: paf->location = APPLY_INT; break;
	case 2: paf->location = APPLY_WIS; break;
	case 3: paf->location = APPLY_CON; break;
	case 4: paf->location = APPLY_DEX; break;
	case 5: paf->location = APPLY_CHA; break;
	case 6: paf->location = APPLY_LCK; break;
	}
        paf->modifier       = 1;
 	xCLEAR_BITS( paf->bitvector );
        LINK( paf, obj->first_affect, obj->last_affect, next, prev );
	return;
   }

   if( !str_cmp( argument, "object2" ) )
   {
	OBJ_DATA *obj;
	OBJ_INDEX_DATA *pObjIndex;
	AFFECT_DATA *paf;

	if( ch->pcdata->favor < ch->pcdata->deity->sdeityobj2 )
	{
	   send_to_char( "You are not favored enough for that.\n\r", ch );
	   return;
	}

	pObjIndex = get_obj_index( ch->pcdata->deity->deityobj2 );
	if ( pObjIndex == NULL )
	{
	   send_to_char( "Your deity has no object to pray for.\n\r", ch );
	   return;
	} 
		
	if( !( obj = create_object( pObjIndex, ch->level ) ) )
      {
         log_printf( "create_object: %s:%s, line %d.", __FILE__, __FUNCTION__, __LINE__ );
         return;
      }
	if ( CAN_WEAR( obj, ITEM_TAKE ) )
	   obj = obj_to_char( obj, ch );
	else
	   obj = obj_to_room( obj, ch->in_room, ch );

      stralloc_printf( &obj->name, "sigil %s", ch->pcdata->deity->name );
 
	act( AT_MAGIC, "$n weaves $p from divine matter!", ch, obj, NULL, TO_ROOM );
	act( AT_MAGIC, "You weave $p from divine matter!", ch, obj, NULL, TO_CHAR );
  	ch->pcdata->favor -= ch->pcdata->deity->sdeityobj2;
      adjust_favor( ch, -1, 1 );

        CREATE( paf, AFFECT_DATA, 1 );
        paf->type           = -1;
        paf->duration       = -1;
	switch( ch->pcdata->deity->objstat )
	{
	case 0: paf->location = APPLY_STR; break;
	case 1: paf->location = APPLY_INT; break;
	case 2: paf->location = APPLY_WIS; break;
	case 3: paf->location = APPLY_CON; break;
	case 4: paf->location = APPLY_DEX; break;
	case 5: paf->location = APPLY_CHA; break;
	case 6: paf->location = APPLY_LCK; break;
	}
        paf->modifier       = 1;
 	xCLEAR_BITS( paf->bitvector );
        LINK( paf, obj->first_affect, obj->last_affect, next, prev );
	return;
   }

   if( !str_cmp( argument, "recall" ) )
   {
	ROOM_INDEX_DATA *location;

	if( ch->pcdata->favor < ch->pcdata->deity->srecall )
	{
	   send_to_char( "Your favor is inadequate for such a supplication.\n\r", ch );
 	   return;
	}

	if( IS_ROOM_FLAG( ch->in_room, ROOM_NOSUPPLICATE ) )
	{
	   send_to_char( "You have been forsaken!\n\r", ch );
	   return;
	}

	if( get_timer( ch, TIMER_RECENTFIGHT ) > 0 && !IS_IMMORTAL(ch) )
        {
          send_to_char( "You cannot supplicate recall under adrenaline!\n\r", ch );
          return;
        }

	location = NULL;
 
      /* NOTE: Any changes to the recall code here need to be reflected in standard recall as well */
      if( !IS_NPC(ch) && !location ) 
      {
         if( ch->pcdata->clan )
            location = get_room_index( ch->pcdata->clan->recall );
         if( !location && ch->pcdata->home )
            location = get_room_index( ch->pcdata->home );
      }
 
      if( !location )
    {
         if( ch->in_room->area->continent == ACON_ALSHEROK )
	{
	   location = get_room_index( ch->pcdata->alsherok );

	   if( !location )
		location = get_room_index( ROOM_VNUM_TEMPLE );
	}

         if( ch->in_room->area->continent == ACON_ELETAR )
	{
	   location = get_room_index( ch->pcdata->eletar );

	   if( !location )
	      location = get_room_index( ROOM_VNUM_ELETAR_RECALL );
	}

         if( ch->in_room->area->continent == ACON_ALATIA )
	{
	   location = get_room_index( ch->pcdata->alatia );

	   if( !location )
	      location = get_room_index( ROOM_VNUM_ALATIA_RECALL );
	}
	   if( ch->in_room->area->continent == ACON_ASTRAL )
	   location = get_room_index( astral_target );
    }
	if( ch->pcdata->deity->recallroom )
	 location = get_room_index( ch->pcdata->deity->recallroom );

	if( !location )
       	  location = get_room_index( ROOM_VNUM_TEMPLE );
 
    	if( !location )
    	{
          send_to_char( "You are completely lost.\n\r", ch );
          return;
	}

    	act( AT_MAGIC, "$n disappears in a column of divine power.", ch, NULL, NULL, TO_ROOM );

      leave_map( ch, NULL, location );

    	act( AT_MAGIC, "$n appears in the room from a column of divine mist.", ch, NULL, NULL, TO_ROOM );
    	ch->pcdata->favor -= ch->pcdata->deity->srecall; 
      adjust_favor( ch, -1, 1 );
    	return;
    }
    send_to_char( "You cannot supplicate for that.\n\r", ch );
    return;
}
