/****************************************************************************
 *                   ^     +----- |  / ^     ^ |     | +-\                  *
 *                  / \    |      | /  |\   /| |     | |  \                 *
 *                 /   \   +---   |<   | \ / | |     | |  |                 *
 *                /-----\  |      | \  |  v  | |     | |  /                 *
 *               /       \ |      |  \ |     | +-----+ +-/                  *
 ****************************************************************************
 * AFKMud Copyright 1997-2003 by Roger Libiez (Samson),                     *
 * Levi Beckerson (Whir), Michael Ward (Tarl), Erik Wolfe (Dwip),           *
 * Cameron Carroll (Cam), Cyberfox, Karangi, Rathian, Raine, and Adjani.    *
 * All Rights Reserved.                                                     *
 *                                                                          *
 * Original SMAUG 1.4a written by Thoric (Derek Snider) with Altrag,        *
 * Blodkai, Haus, Narn, Scryn, Swordbearer, Tricops, Gorog, Rennard,        *
 * Grishnakh, Fireblade, and Nivek.                                         *
 *                                                                          *
 * Original MERC 2.1 code by Hatchet, Furey, and Kahn.                      *
 *                                                                          *
 * Original DikuMUD code by: Hans Staerfeldt, Katja Nyboe, Tom Madsen,      *
 * Michael Seifert, and Sebastian Hammer.                                   *
 ****************************************************************************
 *                            Line Editor Code                              *
 ****************************************************************************/

#include <stdarg.h>
#include <string.h>
#include "mud.h"
#include "editor.h"

void editor_print_info( CHAR_DATA *ch, EDITOR_DATA *edd, sh_int max_size, char *argument )
{
	ch_printf( ch, "Currently editing: %s\n\r"
		"Total lines: %4d   On line:  %4d\n\r"
		"Buffer size: %4d   Max size: %4d\n\r",
		edd->desc ? edd->desc : "(Null description)",
		edd->numlines, edd->on_line, edd->size, max_size );
}

void set_editor_desc( CHAR_DATA *ch, char *new_desc )
{
	if( !ch || !ch->pcdata->editor )
		return;

		STRFREE( ch->pcdata->editor->desc );
	ch->pcdata->editor->desc = STRALLOC( new_desc );
}

void editor_desc_printf( CHAR_DATA *ch, char *desc_fmt, ... )
{
   char buf[MSL*2]; /* umpf.. */
	va_list args;

   va_start( args, desc_fmt );
   vsnprintf( buf, MSL*2, desc_fmt, args );
   va_end( args );
	
	set_editor_desc( ch, buf );
}

/*
 * Removes the tildes from a line, except if it's the last character.
 */
void smush_tilde( char *str )
{
    int len;
    char last;
    char *strptr;
    
    strptr = str;
    
    len  = strlen( str );
    if ( len )
      last = strptr[len-1];
    else
      last = '\0';

    for ( ; *str != '\0'; str++ )
    {
	if ( *str == '~' )
	    *str = '-';
    }
    if ( len )
      strptr[len-1] = last;

    return;
}

void stop_editing( CHAR_DATA *ch )
{
    set_char_color( AT_PLAIN, ch );
    DISPOSE( ch->pcdata->editor );
    send_to_char( "Done.\n\r", ch );
    ch->pcdata->dest_buf  = NULL;
    ch->pcdata->spare_ptr = NULL;
    ch->substate  = SUB_NONE;
    if ( !ch->desc )
    {
	bug( "%s", "Fatal: stop_editing: no desc" );
	return;
    }
    ch->desc->connected = CON_PLAYING;
}

void start_editing( CHAR_DATA *ch, char *data )
{
	EDITOR_DATA *edit;
	sh_int lines, size, lpos;
	char c;
	
	if ( !ch->desc )
	{
	   bug( "%s", "Fatal: start_editing: no desc" );
	   return;
	}
	if ( ch->substate == SUB_RESTRICTED )
	   bug( "%s", "NOT GOOD: start_editing: ch->substate == SUB_RESTRICTED" );

	set_char_color( AT_GREEN, ch );
	send_to_char( "Begin entering your text now (/? = help /s = save /c = clear /l = list)\n\r", ch );
	send_to_char( "-----------------------------------------------------------------------\n\r> ", ch );
	if ( ch->pcdata->editor )
	  stop_editing( ch );
	
	CREATE( edit, EDITOR_DATA, 1 );
	edit->numlines = 0;
	edit->on_line  = 0;
	edit->size     = 0;
	size = 0;  lpos = 0;  lines = 0;
	if ( !data )
	    bug( "%s", "editor: data is NULL!" );
	else
	for ( ;; )
	{  
	   c = data[size++];
	   if ( c == '\0' )
	   {
		edit->line[lines][lpos] = '\0';
		break;
	   }
	   else
	   if ( c == '\r' );
	   else
	   if ( c == '\n' || lpos > 78)
	   {
		edit->line[lines][lpos] = '\0';
		++lines;
		lpos = 0;
	   }
	   else
	     edit->line[lines][lpos++] = c;
	   if ( lines >= 49 || size > 4096 )
	   {
		edit->line[lines][lpos] = '\0';
		break;
	   }	   
	}
	if ( lpos > 0 && lpos < 78 && lines < 49 )
	{
	    edit->line[lines][lpos] = '~';
	    edit->line[lines][lpos+1] = '\0';
	    ++lines;
	    lpos = 0;
	}
	edit->numlines = lines;
	edit->size = size;
	edit->on_line = lines;
	ch->pcdata->editor = edit;
	ch->desc->connected = CON_EDITING;
}

char *copy_buffer( CHAR_DATA *ch )
{
   char buf[MSL];
   char tmp[100];
   sh_int x, len;

   if ( !ch )
   {
	bug( "%s", "copy_buffer: null ch" );
	return STRALLOC( "" );
   } 

   if ( !ch->pcdata->editor )
   {
	bug( "%s", "copy_buffer: null editor" );
	return STRALLOC( "" );
   }

   buf[0] = '\0';
   for ( x = 0; x < ch->pcdata->editor->numlines; x++ )
   {
      mudstrlcpy( tmp, ch->pcdata->editor->line[x], 100 );
      len = strlen(tmp);
      if( tmp && tmp[len-1] == '~' )
	tmp[len-1] = '\0';
      else
	   mudstrlcat( tmp, "\n\r", 100 );
      smash_tilde(tmp);
      mudstrlcat( buf, tmp, 100 );
   }
   return STRALLOC( buf );
}

char *copy_buffer_nohash( CHAR_DATA *ch )
{
   char buf[MSL];
   char tmp[100];
   sh_int x, len;

   if ( !ch )
   {
	bug( "%s", "copy_buffer_nohash: null ch" );
	return str_dup( "" );
   } 

   if ( !ch->pcdata->editor )
   {
	bug( "%s", "copy_buffer_nohash: null editor" );
	return str_dup( "" );
   }

   buf[0] = '\0';
   for ( x = 0; x < ch->pcdata->editor->numlines; x++ )
   {
      mudstrlcpy( tmp, ch->pcdata->editor->line[x], 100 );
      len = strlen(tmp);
      if( tmp && tmp[len-1] == '~' )
	tmp[len-1] = '\0';
      else
	   mudstrlcat( tmp, "\n\r", 100 );
      smash_tilde(tmp);
      mudstrlcat( buf, tmp, 100 );
   }
   return str_dup( buf );
}

/*
 * Simple but nice and handy line editor.			-Thoric
 */
void edit_buffer( CHAR_DATA *ch, char *argument )
{
    DESCRIPTOR_DATA *d;
    EDITOR_DATA *edit;
    char cmd[MIL];
    char buf[MIL];
    sh_int x, line, max_buf_lines;
    bool save;

    if ( (d = ch->desc) == NULL )
    {
	send_to_char( "You have no descriptor.\n\r", ch );
	return;
    }

    if ( d->connected != CON_EDITING )
    {
	send_to_char( "You can't do that!\n\r", ch );
	bug( "%s", "Edit_buffer: d->connected != CON_EDITING" );
	return;
    }
    
    if ( ch->substate <= SUB_PAUSE )
    {
	send_to_char( "You can't do that!\n\r", ch );
	bug( "Edit_buffer: illegal ch->substate (%d)", ch->substate );
	d->connected = CON_PLAYING;
	return;
    }
   
    if ( !ch->pcdata->editor )
    {
	send_to_char( "You can't do that!\n\r", ch );
	bug( "%s", "Edit_buffer: null editor" );
	d->connected = CON_PLAYING;
	return;
    }
   
    edit = ch->pcdata->editor;
    save = FALSE;
    max_buf_lines = 50;

    if ( ch->substate == SUB_MPROG_EDIT || ch->substate == SUB_HELP_EDIT )
	max_buf_lines = 60;

    if ( argument[0] == '/' || argument[0] == '\\' )
    {
	one_argument( argument, cmd );
	if ( !str_cmp( cmd+1, "?" ) )
	{
	    send_to_char( "Editing commands\n\r---------------------------------\n\r", ch );
	    send_to_char( "/l              list buffer\n\r",	ch );
	    send_to_char( "/c              clear buffer\n\r",	ch );
	    send_to_char( "/d [line]       delete line\n\r",	ch );
	    send_to_char( "/g <line>       goto line\n\r",	ch );
	    send_to_char( "/i <line>       insert line\n\r",	ch );
	    send_to_char( "/f <format>     format text in buffer\n\r", ch );
	    send_to_char( "/r <old> <new>  global replace\n\r",	ch );
	    send_to_char( "/a              abort editing\n\r",	ch );
	    send_to_char( "/p              show buffer information\n\r", ch );

	    if ( get_trust(ch) > LEVEL_IMMORTAL )
		send_to_char( "/! <command>    execute command (do not use another editing command)\n\r",  ch );
	    send_to_char( "/s              save buffer\n\r\n\r> ",ch );
	    return;
	}
	if ( !str_cmp( cmd+1, "c" ) )
	{
	    memset( edit, '\0', sizeof(EDITOR_DATA) );
	    edit->numlines = 0;
	    edit->on_line   = 0;
	    send_to_char( "Buffer cleared.\n\r> ", ch );
	    return;
	}
	if ( !str_cmp( cmd+1, "r" ) )
	{
	    char word1[MIL];
	    char word2[MIL];
	    char *sptr, *wptr, *lwptr;
	    int count, wordln, word2ln, lineln;

	    sptr = one_argument( argument, word1 );
	    sptr = one_argument( sptr, word1 );
	    sptr = one_argument( sptr, word2 );
	    if ( word1[0] == '\0' || word2[0] == '\0' )
	    {
		send_to_char( "Need word to replace, and replacement.\n\r> ", ch );
		return;
	    }
	    if ( !str_cmp( word1, word2 ) )
	    {
		send_to_char( "Done.\n\r> ", ch );
		return;
	    }
	    count = 0;  wordln = strlen(word1);  word2ln = strlen(word2);
	    ch_printf( ch, "Replacing all occurrences of %s with %s...\n\r", word1, word2 );
	    for ( x = edit->on_line; x < edit->numlines; x++ )
	    {
		lwptr = edit->line[x];
		while ( (wptr = strstr( lwptr, word1 )) != NULL )
		{
		    ++count;
		    lineln = snprintf( buf, MIL, "%s%s", word2, wptr + wordln );
		    if (lineln + wptr - edit->line[x] > 79)
			buf[lineln] = '\0';
		    mudstrlcpy( wptr, buf, 81 - (wptr - lwptr) );
		    lwptr = wptr + word2ln;
		}
	    }
	    ch_printf( ch, "Found and replaced %d occurrence(s).\n\r> ", count );
	    return;
	}

      /* added format command - shogar */
	/* This has been redone to be more efficient, and to make format
	   start at beginning of buffer, not whatever line you happened
	   to be on, at the time.   */

      if ( !str_cmp( cmd+1, "f" ) )
	{
	    char   temp_buf[MSL+max_buf_lines];
	    int    ep, old_p, end_mark; 
	    int    p = 0;
	    
          send_to_pager( "Reformating...\n\r", ch );

	    for( x = 0; x < edit->numlines; x++ )
	    {
	       mudstrlcpy( temp_buf+p, edit->line[x], MSL+max_buf_lines - p );
	       p += strlen( edit->line[x] );
	       temp_buf[p] = ' ';
	       p++;
	    }
	    
	    temp_buf[p] = '\0';
	    end_mark = p;
	    p = 75;
	    old_p =0;
	    edit->on_line =0;
	    edit->numlines =0;
	    
	    while ( old_p < end_mark )
	    {
	        while ( temp_buf[p] != ' ' && p > old_p )
	          p--;
	          
	        if ( p == old_p )
	          p +=75;
	        
	        if ( p > end_mark )
	          p = end_mark;
	        
	        ep =0;
	        for ( x= old_p ; x < p ; x++ )  
	        {
	           edit->line[edit->on_line][ep] = temp_buf[x];
                 ep++;
              }
              edit->line[edit->on_line][ep] = '\0';

	        edit->on_line++;
	        edit->numlines++;
	        
	        old_p = p+1 ;
	        p += 75;
	        
	    }
          send_to_pager( "Reformating done.\n\r> ", ch );
	    return;
	}

	if ( !str_cmp( cmd+1, "p" ) )
	{
	    editor_print_info( ch, edit, max_buf_lines, argument );
	    return;
	}
 
	if ( !str_cmp( cmd+1, "i" ) )
	{
	    if ( edit->numlines >= max_buf_lines )
		send_to_char( "Buffer is full.\n\r> ", ch );
	    else
	    {
		if ( argument[2] == ' ' )
		  line = atoi( argument + 2 ) - 1;
		else
		  line = edit->on_line;
		if ( line < 0 )
		  line = edit->on_line;
		if ( line < 0 || line > edit->numlines )
		  send_to_char( "Out of range.\n\r> ", ch );
		else
		{
		  for ( x = ++edit->numlines; x > line; x-- )
			mudstrlcpy( edit->line[x], edit->line[x-1], 81 );
		  mudstrlcpy( edit->line[line], "", 81 );
		  send_to_char( "Line inserted.\n\r> ", ch );
		}
 	    }
	    return;
	}

	if ( !str_cmp( cmd+1, "d" ) )
	{
	    if ( edit->numlines == 0 )
		send_to_char( "Buffer is empty.\n\r> ", ch );
	    else
	    {
		if ( argument[2] == ' ' )
		    line = atoi( argument + 2 ) - 1;
		else
		    line = edit->on_line;
		if ( line < 0 )
		    line = edit->on_line;
		if ( line < 0 || line > edit->numlines )
		    send_to_char( "Out of range.\n\r> ", ch );
		else
		{
		    if ( line == 0 && edit->numlines == 1 )
		    {
			memset( edit, '\0', sizeof(EDITOR_DATA) );
			edit->numlines = 0;
			edit->on_line   = 0;
			send_to_char( "Line deleted.\n\r> ", ch );
			return;
		    }
		    for ( x = line; x < (edit->numlines - 1); x++ )
			mudstrlcpy( edit->line[x], edit->line[x+1], 81 );
		    mudstrlcpy( edit->line[edit->numlines--], "", 81 );
		    if ( edit->on_line > edit->numlines )
			edit->on_line = edit->numlines;
		    send_to_char( "Line deleted.\n\r> ", ch );
		}
 	    }
	    return;
	}
	if ( !str_cmp( cmd+1, "g" ) )
	{
	    if ( edit->numlines == 0 )
		send_to_char( "Buffer is empty.\n\r> ", ch );
	    else
	    {
		if ( argument[2] == ' ' )
		    line = atoi( argument + 2 ) - 1;
		else
		{
		    send_to_char( "Goto what line?\n\r> ", ch );
		    return;
		}
		if ( line < 0 )
		    line = edit->on_line;
		if ( line < 0 || line > edit->numlines )
		    send_to_char( "Out of range.\n\r> ", ch );
		else
		{
		    edit->on_line = line;
		    ch_printf( ch, "(On line %d)\n\r> ", line+1 );
		}
 	    }
	    return;
	}
	if ( !str_cmp( cmd+1, "l" ) )
	{
	    if ( edit->numlines == 0 )
		send_to_char( "Buffer is empty.\n\r> ", ch );
	    else
	    {
		send_to_char( "------------------\n\r", ch );
		for ( x = 0; x < edit->numlines; x++ )
		    ch_printf( ch, "%2d> %s\n\r", x+1, edit->line[x] );
		send_to_char( "------------------\n\r> ", ch );
	    }
	    return;
	}
      if( !str_cmp( cmd+1, "a" ) )
      {
         if( !ch->last_cmd )
	{
	    stop_editing( ch );
	    return;
	}
         d->connected = CON_PLAYING;
         ch->substate = SUB_EDIT_ABORT;
         DISPOSE( ch->pcdata->editor );
         send_to_char( "Done.\n\r", ch );
         ch->pcdata->dest_buf  = NULL;
         ch->pcdata->spare_ptr = NULL;
         (*ch->last_cmd) (ch, "" );
         return;
      }
	if ( get_trust(ch) > LEVEL_IMMORTAL && !str_cmp( cmd+1, "!" ) )
	{
	    DO_FUN *last_cmd;
	    int substate = ch->substate;

	    last_cmd = ch->last_cmd;
	    ch->substate = SUB_RESTRICTED;
	    interpret( ch, argument + 3 );
	    ch->substate = substate;
	    ch->last_cmd = last_cmd;
	    set_char_color( AT_GREEN, ch );
	    send_to_char( "\n\r> ", ch );
	    return;
	}
	if ( !str_cmp( cmd+1, "s" ) )
	{
	    d->connected = CON_PLAYING;
	    if ( !ch->last_cmd )
	      return;

	    (*ch->last_cmd) ( ch, "" );
	    return;
	}
    }

    if ( edit->size + strlen(argument) + 1 >= MSL - 1 )
	send_to_char( "You buffer is full.\n\r", ch );
    else
    {
	if ( strlen(argument) > 79 )
	{
	  mudstrlcpy( buf, argument, 80 );
	  send_to_char( "(Long line trimmed)\n\r> ", ch );
	}
	else
	  mudstrlcpy( buf, argument, MIL );
	mudstrlcpy( edit->line[edit->on_line++], buf, 81 );
	if ( edit->on_line > edit->numlines )
	  edit->numlines++;
	if ( edit->numlines > max_buf_lines )
	{
	  edit->numlines = max_buf_lines;
	  send_to_char( "Buffer full.\n\r", ch );
	  save = TRUE;
	}
    }

    if ( save )
    {
	d->connected = CON_PLAYING;
	if ( !ch->last_cmd )
	    return;
	(*ch->last_cmd) ( ch, "" );
	return;
    }
    send_to_char( "> ", ch );
}
