/****************************************************************************
 *                   ^     +----- |  / ^     ^ |     | +-\                  *
 *                  / \    |      | /  |\   /| |     | |  \                 *
 *                 /   \   +---   |<   | \ / | |     | |  |                 *
 *                /-----\  |      | \  |  v  | |     | |  /                 *
 *               /       \ |      |  \ |     | +-----+ +-/                  *
 ****************************************************************************
 * AFKMud Copyright 1997-2003 by Roger Libiez (Samson),                     *
 * Levi Beckerson (Whir), Michael Ward (Tarl), Erik Wolfe (Dwip),           *
 * Cameron Carroll (Cam), Cyberfox, Karangi, Rathian, Raine, and Adjani.    *
 * All Rights Reserved.                                                     *
 *                                                                          *
 * Original SMAUG 1.4a written by Thoric (Derek Snider) with Altrag,        *
 * Blodkai, Haus, Narn, Scryn, Swordbearer, Tricops, Gorog, Rennard,        *
 * Grishnakh, Fireblade, and Nivek.                                         *
 *                                                                          *
 * Original MERC 2.1 code by Hatchet, Furey, and Kahn.                      *
 *                                                                          *
 * Original DikuMUD code by: Hans Staerfeldt, Katja Nyboe, Tom Madsen,      *
 * Michael Seifert, and Sebastian Hammer.                                   *
 ****************************************************************************
 *                             Event Q Processor                            *
 ****************************************************************************/

/*
 * Event.c
 *
 * Primary Code for Event Handling & Management (Insanity v0.9a pre-release)
 * Written by Chris Fewtrell <C.J.Fewtrell@bcs.org.uk>
 * 14 November 2000 (With many adjustments thereafter)
 *
 * Adjustments Made For AFKMud
 * 2nd March 2002
 *
 * This file contains all the main functions and code to run the event q
 * it is relativly self contained. Mess with this code and you *could*
 * seriously affect the performance of your mud (due to the frequency
 * calls made to it from the main update_handler) so treat with care.
 *
 * Adjustments made for AFKMud at request of Samson@Alsherok
 *
 * Based on texts and code fragments by Erwin Andresean and 
 * Oliver Jowett.
 *
 * Please retain all credit for this code within this file.
 */

#include "mud.h"
#include "event.h"

#define EVENT_HASH 512

EVENT *event_table[EVENT_HASH];

int top_event;
int top_free_event;
int executed_free=0;
bool activeMprogs = TRUE;
bool activeActions = TRUE;
extern void program_flow ( EVENT* event );


struct hook_type misc_ev_hooks[] =
{

    { "",			ev_null,		NULL,			FALSE }
};

struct hook_type temp_ev_hooks[] =
{
    { "ev_violence",		evt_violence,		ev_violence,		TRUE	},
    { "ev_area_reset",		evt_area_reset,		ev_area_reset,		TRUE	},
    { "ev_auction",		evt_auction,		ev_auction,			TRUE  },
    { "ev_reboot",		evt_reboot,			ev_reboot_count,		TRUE  },
    { "",			ev_null,		NULL,			FALSE	}
};

EVENT *find_event( EVENT **eventlist, int type, int subtype )
{
   EVENT *e;

   for( e = *eventlist; e && !( e->type == type && e->subtype == subtype ); e = e->next_local )
      if( !e )
	   return NULL;

   return( e );
}

EVENT *new_event( void )
{
   EVENT *enew;

   CREATE( enew, EVENT, 1 );

   enew->type = EV_NULL;
   enew->next_local = NULL;
   enew->next_global = NULL;
   enew->flags = 0;
   enew->actor_type = 0;
   enew->deleted = FALSE;
    
   return( enew );
}

void free_event( EVENT *old )
{
   if( !old ) /* Sanity Check */
	return;

   if( IS_SET( old->flags, EVF_FREE ) )
    {
	log_string( "Attempting to free a marked free event" );
	abort();
    }

    old->next_global = NULL;

    /* Per Type Memory handling */
   switch( old->type )
    {
        case EV_MPROG:
         STRFREE( old->data.mprog.arguments );

         if( old->data.mprog.scripting && old->data.mprog.enviroment )
            DISPOSE( old->data.mprog.enviroment );
            break;

	case EV_TEMP:
         switch( old->subtype )
            { 
		default:
		    break;
            }
	    break;

        case EV_ACTION:
         STRFREE( old->data.action.arguments );
            break;
    }
   DISPOSE( old );
    return;
}

void add_event( EVENT *enew, int when )
{
   EVENT *event, *event_prev = NULL;
    int hash;

   if( IS_SET( enew->flags, EVF_ADDED ) )
    {
	log_string( "----> Attempted to double add a statement" );
	return;
    }

   if( IS_SET( enew->flags, EVF_FREE ) )
    {
	log_string("----> Attempting to schedule a free event");
	return;
    }

/*
    if(when < sysdata.pulses)
	return;
*/

    hash = when % EVENT_HASH;
    
   enew->when = when;

   SET_BIT( enew->flags, EVF_ADDED );

   for( event = event_table[hash]; event; event = event->next_global )
    {
	if( event->when >= when )
 	{
	   if( event_prev )
		event_prev->next_global = enew;
	    else
		event_table[hash] = enew;
	
	   enew->next_global = event;
    	   /* log_printf( "Added event with when at %d", when ); */
	    return;
	}
    event_prev = event;
    }

   if( event_prev )
	event_prev->next_global = enew;
    else
	event_table[hash] = enew;

   /* log_printf( "Added event with when at %d", when ); */
    return;
}

void remove_event( EVENT* delevent )
{
    EVENT *event, *event_prev = NULL;
    int hash = delevent->when % EVENT_HASH;

    if(!IS_SET(delevent->flags, EVF_ADDED))
	return;


    for(event = event_table[hash] ; event ; event = event->next_global )
    {
	if(event == delevent)
	{
	    if(event_prev)
		event_prev->next_global = event->next_global;
	    else
		event_table[hash] = event->next_global;

	    delevent->next_global = NULL;
	    REMOVE_BIT(delevent->flags, EVF_ADDED);
	    return;
	}

	if(event == event_prev)
	{
	    /* ARRRGHHH How'd this happen!?!? */
	    log_string("remove_event: Same event found looping globally");
	    abort();
     	}

	event_prev = event;
    }

    return;
}

void remove_event_local(EVENT **eventlist, EVENT *delevent)
{
    EVENT *event, *event_prev = NULL;

    if(!delevent)
	return;
 
    for( event = *eventlist ; event ; event = event->next_local )
    {
	if( event == delevent )
	{
	    if(event_prev)
		event_prev->next_local = event->next_local;
	    else
		*eventlist = event->next_local;

	    delevent->next_local = NULL;

	    return;
	}

	event_prev = event;
    }

    return;
}

void add_simple_ev( CHAR_DATA *ch, int type, int subtype, int offset, bool fAllowDuplicates )
{
    EVENT *event;

    /* Prevent the addition of duplicates */
    if(!fAllowDuplicates && ch && find_event( &ch->events, type, subtype ))
	return;

    event = new_event();

    event->type = type;
    event->subtype = subtype;

    if(ch)
    {
    	event->actor.mob = ch;
    	event->actor_type = EVA_MOB;
    	event->next_local = ch->events;
    	ch->events = event;
    }
    else
    {
	event->actor_type = EVA_NONE;
    }

    add_event(event, sysdata.pulses+offset);

    return;
}

int count_scheduled_free_events( void )
{
   int bucket = 0, events = 0;
   EVENT *e;

   for( bucket = 0; bucket < EVENT_HASH; bucket++ )
      for( e = event_table[bucket]; e; e = e->next_global )
         if( !e || IS_SET( e->flags, EVF_FREE ) )
            events++;

   return( events );
}

/* 
 * THE core event handler, this code is sensitive, don't event
 * speak harshly at it! Doing so could severly affect the performance
 * of the MUD (or just completely break it)
 */
void event_update( void )
{
    EVENT *event, *event_next;
   int oldtype = 0, oldsubtype = 0;

    int hash = sysdata.pulses % EVENT_HASH;

   for( event = event_table[hash]; event; event = event_next )
    {
	if( event->when > sysdata.pulses )
	{
	    event_table[hash] = event;
	    return; /* If we've got here we have got to now in the sorted list */
   	}

 	event_next = event->next_global;

	if( IS_SET( event->flags, EVF_FREE ) )
	{	
            executed_free++;
         log_printf( "%s", "----> Event_update: Attempting to execute a free event....." );
         log_printf( "----> In bucket %d to execute on pulse %d (Current on %ld)", hash, event->when, sysdata.pulses );
         log_printf( "      Event Details: Type %d Sub-Type: %d Marked Free: %s", event->type, event->subtype,
            IS_SET( event->flags, EVF_FREE ) ? "Yes" : "No" );
         log_printf( "      Stored in Memory Location %ld", (long int)event );
         log_printf( "Found total of %d scheduled free events at this point.", count_scheduled_free_events() );
         log_printf( "%s", "      .....skipping." );
/*	    REMOVE_BIT( event->flags, EVF_FREE ); */
/*	    free_event( event ); */
	   bug( "%s", "Attempting to execute a freed event?" );
	     event_table[hash] = event_next;
	    return;
/*	    continue; */
	}

	if( event->type == EV_NULL || event->deleted )
	{
	   bug( "%s", "NULL Event Type.. or deleted event.. skipping" );
	   free_event( event );
	    event_table[hash] = event_next;
	    continue;
	}

	oldtype = event->type;
	oldsubtype = event->subtype;

	/* Heal the Event Table now, just in case.. */
	event_table[hash] = event_next;

	switch( event->actor_type )
	{
	    case EVA_MOB:
		remove_event_local( &event->actor.mob->events, event );
	   	break;

	    case EVA_OBJ:
		remove_event_local( &event->actor.obj->events, event );
		break;
	
	    case EVA_ROOM:
		remove_event_local( &event->actor.room->events, event );
		break;

	    case EVA_AREA:
		remove_event_local( &event->actor.area->events, event );
		break;

/*
 * Removed for AFKMud Installation, may readd later on again
 *
	    case EVA_EXIT:
		remove_event_local( &event->actor.exit->events, event );
		break;
*
*/
	    default:
		/* Create a default global list for non-local events? */
		break;
	}

	if( oldtype != event->type || oldsubtype != event->subtype )
	   log_printf( "Event changed types during execution? Old type %d subtype %d", oldtype, oldsubtype );

	/*
	 * The event in question has been removed from the local entity in question
 	 * at this point, thus can be processed and deleted from the global list as
	 * well, allowing the event structure to be freed when required
	 * 
	 * Now to proecess based upon event type/subtype using the predefined tables
	 */

	REMOVE_BIT( event->flags, EVF_ADDED );

	switch( event->type )
	{
	     case EV_TEMP:
		if( temp_ev_hooks[event->subtype-1].hook_fun && temp_ev_hooks[event->subtype-1].active )
		{
		    (temp_ev_hooks[event->subtype-1].hook_fun) (event);
		}
		break;

	     case EV_MISC:
		if( misc_ev_hooks[event->subtype-1].hook_fun && misc_ev_hooks[event->subtype-1].active )
		    (misc_ev_hooks[event->subtype-1].hook_fun) (event);
		break;

	     case EV_MPROG:
		/* if( activeMprogs )
		    program_flow( event ); */
		break;
	     case EV_ACTION:
		if( activeActions )
		    (event->data.action.hook) (event);
		break;

	     default:
		bug( "%s", "Wierd, no major event type, that shouldn't be possible here..." );
		break;
	}

	sysdata.events_served++;

	/*
	 * Though the nature of the event tables event is already removed from
	 * its global list
	 */
	free_event( event );

    }

    /*
     * We've fallen out of the other end, must be nothing else in the
     * bucket to be precessed.
     */

    event_table[hash] = NULL;
    return;
}

void add_mprog_ev( int vnum, void *actor, int actor_type, CHAR_DATA *ch, void *arg1, void *arg2, char *arguments )
{
    EVENT* event; 

    event = new_event();
    event->type = EV_MPROG;
    event->subtype = EV_NULL;
     
    /* Setup the actor/executor */

   switch( actor_type )
    {
        case EVA_MOB:
            event->actor.mob = (CHAR_DATA *)actor;
            event->actor_type = EVA_MOB;
            break;
        case EVA_OBJ:
            event->actor.obj = (OBJ_DATA*)actor;
            event->actor_type = EVA_OBJ;
            break;
        case EVA_ROOM:
            event->actor.room = (ROOM_INDEX_DATA*)actor;
            event->actor_type = EVA_ROOM;
            break;
    }
    
    event->data.mprog.vnum = vnum;
    event->data.mprog.trigger = ch;
   event->data.mprog.arg1 = (char *)arg1;
   event->data.mprog.arg2 = (char *)arg2;
   event->data.mprog.arguments = STRALLOC( arguments );
    event->data.mprog.enviroment = NULL;
    event->data.mprog.scripting = FALSE;
    
   switch( event->actor_type )
    {
        case EVA_MOB:
            event->next_local = event->actor.mob->events;
            event->actor.mob->events = event;
            break;   
        case EVA_OBJ:
            event->next_local = event->actor.obj->events;
            event->actor.obj->events = event;
            break;
        case EVA_ROOM:
            event->next_local = event->actor.room->events;
            event->actor.room->events = event;
            break;

        default:
         free_event( event );
            return;
    }
   add_event( event, sysdata.pulses + 1 );
}
    
void run_mprog_ev_now( int vnum, void *actor, int actor_type, CHAR_DATA *ch, void *arg1, void *arg2, char *arguments )
{
    EVENT* event;
            
    event = new_event();
    event->type = EV_MPROG;
    event->subtype = EV_NULL;

    /* Setup the actor/executor */
   switch( actor_type )
    {
        case EVA_MOB:
            event->actor.mob = (CHAR_DATA *)actor;
            event->actor_type = EVA_MOB;
            break;
        case EVA_OBJ:
            event->actor.obj = (OBJ_DATA*)actor;
            event->actor_type = EVA_OBJ;
            break;
        case EVA_ROOM:
            event->actor.room = (ROOM_INDEX_DATA*)actor;
            event->actor_type = EVA_ROOM;
            break;
    }
    
    event->data.mprog.vnum = vnum;
    event->data.mprog.trigger = ch;
   event->data.mprog.arg1 = (char *)arg1;
   event->data.mprog.arg2 = (char *)arg2;
   event->data.mprog.arguments = STRALLOC( arguments );
    event->data.mprog.enviroment = NULL;
    event->data.mprog.scripting = FALSE;
    
   switch( event->actor_type )
    {
        case EVA_MOB:
            event->next_local = event->actor.mob->events;
            event->actor.mob->events = event;
            break;   
        case EVA_OBJ:
            event->next_local = event->actor.obj->events;
            event->actor.obj->events = event;
            break;
        case EVA_ROOM:
            event->next_local = event->actor.room->events;
            event->actor.room->events = event;
            break;

        default:
         free_event( event );
            return;
    }
    
    /* program_flow(event); */
   switch( event->actor_type )
    {
        case EVA_MOB:
         if( event->actor.mob )
            remove_event_local( &event->actor.mob->events, event );
            break;

        case EVA_OBJ:
         if( event->actor.obj )
            remove_event_local( &event->actor.obj->events, event );
            break;

        case EVA_ROOM:
         remove_event_local( &event->actor.room->events, event );
            break;
    }
   free_event( event );
}

void add_walkto_ev( CHAR_DATA *ch, int offset )
{
   add_simple_ev( ch, EV_TEMP, evt_walkto, number_fuzzy(offset), FALSE );
}

int count_events( void )
{
    int bucket=0, events=0;
    EVENT *e;

    for(bucket = 0 ; bucket < EVENT_HASH ; bucket++)
        for(e = event_table[bucket] ; e ; e=e->next_global, events++);

    return(events);
}

void add_simple_room_ev(ROOM_INDEX_DATA *room, int type, int subtype, int offset)
{
     EVENT *event;  

     for(event = room->events ; event ; event=event->next_local)
        if(event->type == type && event->subtype == subtype)
            return;

     if(event)
        remove_event(event);
     else
     {
        event = new_event();   
        event->type = type;
        event->subtype = subtype;
        event->actor.room = room;
        event->actor_type = EVA_ROOM;
        event->next_local = room->events;
        room->events = event;
     }
      
     add_event(event, sysdata.pulses+offset);
        
     return;
}

void add_simple_obj_ev(OBJ_DATA *obj, int type, int subtype, int offset)
{
     EVENT *event;  

     for(event = obj->events ; event ; event=event->next_local)
        if(event->type == type && event->subtype == subtype)
            return;

     if(event)
        remove_event(event);
     else
     {

        event = new_event();
        event->type = type;
        event->subtype = subtype;
        event->actor.obj = obj;
        event->actor_type = EVA_OBJ;
        event->next_local = obj->events;
        obj->events = event;
     }
     
     add_event(event, sysdata.pulses+offset);
     
     return;
}

/*
 * Take a loaded list of events local to object and inject into global
 * tables
 */ 
void inject_obj_ev( OBJ_DATA *obj )
{
    EVENT* event; 
   OBJ_DATA *contains;

   for( event = obj->events; event; event = event->next_local )
    {
      if( IS_SET( event->flags, EVF_LOADING ) || !IS_SET( event->flags, EVF_ADDED ) )
        {
         add_event( event, event->when+sysdata.pulses );
         REMOVE_BIT( event->flags, EVF_LOADING );
        }
    }
            
   for( contains = obj->first_content; contains; contains=contains->next_content )
      inject_obj_ev( contains );
            
}

void inject_char_ev( CHAR_DATA *ch )
{
    EVENT* event;
   OBJ_DATA *obj; 

   for( event = ch->events; event; event = event->next_local )
    {
      if( IS_SET( event->flags, EVF_LOADING ) || !IS_SET( event->flags, EVF_ADDED ) )
        {
/*	   log_printf( "Found and Injecting event %d %d", event->type, event->subtype ); */
         add_event( event, event->when+sysdata.pulses );
         REMOVE_BIT( event->flags, EVF_LOADING );
        }
    }
             
   for( obj = ch->first_carrying ; obj ; obj=obj->next_content )
      inject_obj_ev( obj );
}

void fwrite_event( EVENT* event, FILE* fp )
{
   if( !event || event->type == EV_TEMP || event->type == EV_MPROG || event->type == EV_ACTION )
        return;

   if( event->when < sysdata.pulses )
        return;

   fprintf( fp, "Event %d %d %ld", event->type, event->subtype,
      IS_SET( event->flags, EVF_LOADING ) ? event->when : event->when - sysdata.pulses );

   switch( event->type )
    {
        default:
         fprintf( fp, "%s", "\n" );
            break;
    }
}
 
void fread_event( FILE *fp, EVENT* event )
{
    event->flags = EVF_LOADING;
   event->type = fread_number( fp );
   event->subtype = fread_number( fp );   
   event->when = fread_number( fp );
    
/*    log_printf("Fread Event: Event %d %d %d", event->type, event->subtype, event->when); */

   switch( event->type )
    {
        case EV_MISC:
         switch( event->subtype )
            {  
                default:  
                    break;
            }   
            break;
        default:  
            break;
    }
    return;
}

CMDF do_eventinfo(CHAR_DATA *ch, char *arguments)
{
    CHAR_DATA *victim=NULL;
    EVENT *event, *eventlist=NULL;
    OBJ_DATA *obj=NULL;
    int levents=0;

    if(arguments[0] == '\0')
    {
        int events = count_events();
        int fevents = count_scheduled_free_events();

        ch_printf(ch, "There are currently %d events crossing all 512 buckets.\n\r"
            "There are %d allocated event structures with %d in the free list.\n\r"
            "This leaves a count of %d in use with a discrepacy of %d in favor\n\r"
            "of the event queue count.\n\r\n\r"
            "Since reboot there have been %ld events served, with an average of\n\r"
            "%ld events per pulse (%ld events per second - %ld pulses passed)\n\r"
            "\n\rScheuled free events counted : %d\n\r",

            events, top_event, top_free_event, top_event-top_free_event,
            (events - (top_event-top_free_event)),
            sysdata.events_served, sysdata.events_served/sysdata.pulses,
            sysdata.events_served/(sysdata.pulses/sysdata.pulsepersec), sysdata.pulses, fevents );
            
        return;
    }

    if(!str_cmp(arguments, "help") || !str_cmp(arguments, "?"))
    {
	send_to_char( "evinfo [char/obj]\n\r", ch );
	return;
    }

    if(	( victim = get_char_room(ch, arguments ) ) ||
	( obj = get_obj_list(ch, arguments, ch->first_carrying ) ) ||
	( obj = get_obj_list(ch, arguments, ch->in_room->first_content ) ) ||
	( victim = get_char_world(ch, arguments ) ) ||
	( obj = get_obj_list(ch, arguments, first_object ) ) )
    {
	if(victim)
	{
	    ch_printf(ch, "%s %s (Vnum:%d) - Local Event Information\n\r"
		"System currently running at pulse %ld\n\r",

                IS_NPC(victim)?"Mobile":"Player",
                IS_NPC(victim)?victim->short_descr:victim->name,
                IS_NPC(victim)?victim->pIndexData->vnum:-1,	sysdata.pulses );

	    eventlist = victim->events;
	}
	else if(obj)
	{
    	    ch_printf(ch, "Object %d (%s) - Local Event Information\n\r"
		"System currently running at pulse %ld\n\r", 
		obj->pIndexData->vnum, obj->short_descr, sysdata.pulses );

	    eventlist = obj->events;
	}
	else
	{
	    ch_printf(ch, "Unable to locate '%s' to examine event information\n\r", arguments );
	    return;
	}
    }
    else
    {
        ch_printf(ch, "Unable to locate '%s' to examine event information\n\r", arguments );
	return;
    }

    for(event = eventlist; event; event = event->next_local, levents++)
    {
        switch(event->type)
        {
            case EV_TEMP:
                ch_printf(ch, "Event - Type: TEMP trigger %s, %sinserted into global queue for pulse %d.\n\r",
                   temp_ev_hooks[event->subtype-1].evName, IS_SET(event->flags, EVF_ADDED)  ? "" : "NOT ", event->when);
                break;
        
            case EV_MISC:
                ch_printf(ch, "Event - Type: MISC trigger %s, %sinserted into global queue for pulse %d.\n\r",
                   misc_ev_hooks[event->subtype-1].evName, IS_SET(event->flags, EVF_ADDED)  ? "" : "NOT ", event->when);
                break;
    
            case EV_MPROG:      
                ch_printf(ch, "Event - Type MPROG %sinserted into global queue to execute at pulse %d.\n\r",
                   IS_SET(event->flags, EVF_ADDED)  ? "" : "NOT ", event->when);
                break;
            case EV_ACTION:
                ch_printf(ch, "Event - Type ACTION %sinserted into global queue to execution at pulse %d.\n\r",
                   IS_SET(event->flags, EVF_ADDED)  ? "" : "NOT ", event->when);
                break;
            default:
            case EV_NULL:
                ch_printf(ch, "Event - {R**{YWARNING{R**{x NULL event type %sinserted into global queue to execute at pulse %d.\n\r",
                   IS_SET(event->flags, EVF_ADDED)  ? "" : "NOT ", event->when);
                break;
        }
    }

    ch_printf(ch, "%d local events displayed.\n\r", levents);

    return;
}
