/****************************************************************************
 *                   ^     +----- |  / ^     ^ |     | +-\                  *
 *                  / \    |      | /  |\   /| |     | |  \                 *
 *                 /   \   +---   |<   | \ / | |     | |  |                 *
 *                /-----\  |      | \  |  v  | |     | |  /                 *
 *               /       \ |      |  \ |     | +-----+ +-/                  *
 ****************************************************************************
 * AFKMud Copyright 1997-2003 by Roger Libiez (Samson),                     *
 * Levi Beckerson (Whir), Michael Ward (Tarl), Erik Wolfe (Dwip),           *
 * Cameron Carroll (Cam), Cyberfox, Karangi, Rathian, Raine, and Adjani.    *
 * All Rights Reserved.                                                     *
 *                                                                          *
 * Original SMAUG 1.4a written by Thoric (Derek Snider) with Altrag,        *
 * Blodkai, Haus, Narn, Scryn, Swordbearer, Tricops, Gorog, Rennard,        *
 * Grishnakh, Fireblade, and Nivek.                                         *
 *                                                                          *
 * Original MERC 2.1 code by Hatchet, Furey, and Kahn.                      *
 *                                                                          *
 * Original DikuMUD code by: Hans Staerfeldt, Katja Nyboe, Tom Madsen,      *
 * Michael Seifert, and Sebastian Hammer.                                   *
 ****************************************************************************
 *                             Event Q Processor                            *
 ****************************************************************************/

/*
 * Event.h
 *
 * Primary Event Header
 * Written by Chris Fewtrell <C.J.Fewtrell@bcs.org.uk>
 * 14 November 2000
 *
 * This file contains the definitions for the core event structure.
 * This is the heart of the event system, DO NOT EDIT UNLESS YOU KNOW
 * WHAT YOU ARE DOING!
 *
 * You have been warned
 *
 */

/* Primary Event Types */
#define EV_NULL 	0
#define EV_TEMP 	1
#define EV_MISC	2
#define EV_MPROG	3
#define EV_ACTION	4

/* Event Actor types */
#define EVA_NONE 	0
#define EVA_MOB 	1
#define EVA_OBJ 	2
#define EVA_ROOM 	3
#define EVA_AREA	4
#define EVA_EXIT	5

/* Event Flags */
#define EVF_FREE		1
#define EVF_ADDED		2
#define EVF_LOADING	4

typedef struct event	EVENT;
typedef struct prog_env PROG_ENV; /* Hopefully will quell the errors for now */
#define MAX_NESTED_LEVEL 12

#define D_EVHOOK( fun ) EVHOOK fun
#ifndef S_SPLINT_S
typedef void EVHOOK	args( ( EVENT *event ) );
#endif

/* Event Subtype enumerations */

/* Temporary events (don't save) */
enum { ev_null=0, evt_violence, evt_area_reset, evt_auction, evt_reboot,
	evt_specTrigger, evt_mobScavenge,
	evt_mobWanderFlee, evt_walkto, evt_flying, evt_racial_affects, evt_worldTime, evt_worldWeather, evt_60SecUpdate };

/* Miscellaneous - Non specific events, savable */
enum { evm_null=0, evm_objDecay, evm_affect_decay, evm_regen_innate };

enum { evp_null=0, evp_delay };

struct prog_env
{
    int			state[MAX_NESTED_LEVEL];
    int			cond[MAX_NESTED_LEVEL];
    int			script_pos;
    int			call_level;
};

struct event
{
    /* List management */
   EVENT *next_global;
   EVENT *next_local;

    int when;		/* When we are going off */
    bool deleted;

    union
    {
	CHAR_DATA *ch;
	CHAR_DATA *mob;
	ROOM_INDEX_DATA *room;
	OBJ_DATA *obj;
	AREA_DATA *area;
 	EXIT_DATA *exit;
    } actor;
    int actor_type;

    int flags;

    int type;	/* Event Type */
    int subtype;	/* Sub Type */

    union
    {
	struct
	{
	   void *target;
	    int 		target_type;
	   void *data;
	} temp;

	struct
	{
	   CHAR_DATA *trigger;
	   char *arg1;
	   char *arg2;
	   char *arguments;
	   PROG_ENV *enviroment;
	    bool		scripting;
	    int		vnum;
	    bool		debug;
	} mprog;

	struct
	{
#ifndef S_SPLINT_S
	   EVHOOK *hook;
#endif
	    int		stage;
	   char *arguments;
	   void *arg[3];
	} action;
    } data;
};

struct hook_type
{
    char * evName;
    int type;
#ifndef S_SPLINT_S
    EVHOOK *hook_fun;
#endif
    bool active;
};

void add_simple_ev( CHAR_DATA *ch, int type, int subtype, int offset, bool fAllowDuplicates );
void add_event( EVENT *enew, int when );
EVENT *new_event( void );

#ifndef S_SPLINT_S
D_EVHOOK( ev_violence );
D_EVHOOK( ev_area_reset );
D_EVHOOK( ev_auction );
D_EVHOOK( ev_reboot_count );
#endif
