/****************************************************************************
 *                   ^     +----- |  / ^     ^ |     | +-\                  *
 *                  / \    |      | /  |\   /| |     | |  \                 *
 *                 /   \   +---   |<   | \ / | |     | |  |                 *
 *                /-----\  |      | \  |  v  | |     | |  /                 *
 *               /       \ |      |  \ |     | +-----+ +-/                  *
 ****************************************************************************
 * AFKMud Copyright 1997-2003 by Roger Libiez (Samson),                     *
 * Levi Beckerson (Whir), Michael Ward (Tarl), Erik Wolfe (Dwip),           *
 * Cameron Carroll (Cam), Cyberfox, Karangi, Rathian, Raine, and Adjani.    *
 * All Rights Reserved.                                                     *
 *                                                                          *
 * Original SMAUG 1.4a written by Thoric (Derek Snider) with Altrag,        *
 * Blodkai, Haus, Narn, Scryn, Swordbearer, Tricops, Gorog, Rennard,        *
 * Grishnakh, Fireblade, and Nivek.                                         *
 *                                                                          *
 * Original MERC 2.1 code by Hatchet, Furey, and Kahn.                      *
 *                                                                          *
 * Original DikuMUD code by: Hans Staerfeldt, Katja Nyboe, Tom Madsen,      *
 * Michael Seifert, and Sebastian Hammer.                                   *
 ****************************************************************************
 *	                     Overland ANSI Map Module                         *
 *                      Created by Samson of Alsherok                       *
 ****************************************************************************/

#define OVERLAND_ALSHEROK 50000
#define OVERLAND_ALATIA 50500
#define OVERLAND_ELETAR 51000

#define MAX_X 1000
#define MAX_Y 1000

/* Change these filenames to match yours */
#define MAP_DIR "../maps/"
#define ENTRANCE_FILE "entrances.dat"
#define LANDMARK_FILE "landmarks.dat"
#define MAPRESET_FILE "mapresets.dat"

typedef enum
{
   MAP_ALSHEROK, MAP_ELETAR, MAP_ALATIA, MAP_MAX
} map_types;

typedef enum
{
   TYPE_OBJECT, TYPE_MOBILE
} mapreset_types;

/* 3 maps, each of 1000x1000 rooms, starting from 0,0 as the NW corner */
extern char map_sector[MAP_MAX][MAX_X][MAX_Y];
extern char *const map_names[];
extern char *const map_name[];
extern char	*const continents[];
extern char *const sect_types[];
extern const struct sect_color_type sect_show[];

typedef struct	entrance_data	ENTRANCE_DATA;
typedef struct	landmark_data	LANDMARK_DATA;
typedef struct	mapreset_data	MAPRESET_DATA;

extern ENTRANCE_DATA *first_entrance;
extern ENTRANCE_DATA *last_entrance;
extern LANDMARK_DATA *first_landmark;
extern LANDMARK_DATA *last_landmark;
extern MAPRESET_DATA *first_mapreset;
extern MAPRESET_DATA *last_mapreset;

struct mapreset_data
{
  MAPRESET_DATA * next;
  MAPRESET_DATA * prev;
  int type; /* Reset a mob or object */
  int vnum; /* Vnum of object OR mobile */
   sh_int map; /* Map for the reset */
   sh_int x;   /* X coordinate of reset */
   sh_int y;   /* Y coordinate of reset */
};

struct landmark_data
{
   LANDMARK_DATA *next;
   LANDMARK_DATA *prev;
   sh_int map; 	    /* Map the landmark is on */
   sh_int x;   	    /* X coordinate of landmark */
   sh_int y;   	    /* Y coordinate of landmark */
  int	distance; 	   /* Distance the landmark is visible from */
  char *description; /* Description of the landmark */
  bool Isdesc;       /* If true is room desc. If not is landmark */
};

struct entrance_data
{
   ENTRANCE_DATA *next;
   ENTRANCE_DATA *prev;
   sh_int herex;      /* Coordinates the entrance is at */
   sh_int herey;
   sh_int therex;     /* Coordinates the entrance goes to, if any */
   sh_int therey;
   sh_int tomap; 	    /* Map it goes to, if any */
   sh_int onmap; 	    /* Which map it's on */
  int vnum; 	   /* Target vnum if it goes to a regular zone */
  sh_int prevsector; /* Previous sector type to restore with when an exit is deleted */
  char *area;	   /* Area name */
};

struct sect_color_type
{
   sh_int sector; /* Terrain sector */
   char *color; 	/* Color to display as */
   char *symbol; 	/* Symbol you see for the sector */
   char *desc; 	/* Description of sector type */
  bool canpass; 	/* Impassable terrain */
  int move; 	/* Movement loss */
   sh_int graph1;	/* Color numbers for graphic conversion */
   sh_int graph2;
   sh_int graph3;
};

sh_int get_terrain( sh_int map, sh_int x, sh_int y );
ENTRANCE_DATA *check_entrance( sh_int map, sh_int x, sh_int y );
void delete_entrance( ENTRANCE_DATA *enter );
void putterr( sh_int map, sh_int x, sh_int y, sh_int terr );
ch_ret process_exit( CHAR_DATA *ch, sh_int map, sh_int x, sh_int y, int dir, bool running );
double distance( sh_int chX, sh_int chY, sh_int lmX, sh_int lmY );
double calc_angle( sh_int chX, sh_int chY, sh_int lmX, sh_int lmY, double *ipDistan );
