/****************************************************************************
 *                   ^     +----- |  / ^     ^ |     | +-\                  *
 *                  / \    |      | /  |\   /| |     | |  \                 *
 *                 /   \   +---   |<   | \ / | |     | |  |                 *
 *                /-----\  |      | \  |  v  | |     | |  /                 *
 *               /       \ |      |  \ |     | +-----+ +-/                  *
 ****************************************************************************
 * AFKMud Copyright 1997-2003 by Roger Libiez (Samson),                     *
 * Levi Beckerson (Whir), Michael Ward (Tarl), Erik Wolfe (Dwip),           *
 * Cameron Carroll (Cam), Cyberfox, Karangi, Rathian, Raine, and Adjani.    *
 * All Rights Reserved.                                                     *
 *                                                                          *
 * Original SMAUG 1.4a written by Thoric (Derek Snider) with Altrag,        *
 * Blodkai, Haus, Narn, Scryn, Swordbearer, Tricops, Gorog, Rennard,        *
 * Grishnakh, Fireblade, and Nivek.                                         *
 *                                                                          *
 * Original MERC 2.1 code by Hatchet, Furey, and Kahn.                      *
 *                                                                          *
 * Original DikuMUD code by: Hans Staerfeldt, Katja Nyboe, Tom Madsen,      *
 * Michael Seifert, and Sebastian Hammer.                                   *
 ****************************************************************************
 *                          Shaddai's Polymorph                             *
 ****************************************************************************/

#define MORPH_FILE      "morph.dat"     /* For morph data */

typedef struct char_morph CHAR_MORPH;
typedef struct morph_data MORPH_DATA;

extern MORPH_DATA *morph_start;
extern MORPH_DATA *morph_end;

int do_morph_char( CHAR_DATA *ch, MORPH_DATA *morph );
void do_unmorph_char( CHAR_DATA *ch );

#define MORPHPERS(ch, looker, from) ( can_see( (looker), (ch), (from) ) ? (ch)->morph->morph->short_desc : "Someone" )

/*
 * Structure for a morph -- Shaddai
 */
/*
 *  Morph structs.
 */

#define ONLY_PKILL  	1
#define ONLY_PEACEFULL  2

struct char_morph
{
   MORPH_DATA *morph;
   EXT_BV affected_by;    /* New affected_by added */
   EXT_BV no_affected_by; /* Prevents affects from being added */
   int no_immune;      /* Prevents Immunities */
   int no_resistant;   /* Prevents resistances */
   int no_suscept;     /* Prevents Susceptibilities */
   int resistant;      /* Resistances added */
   int suscept;        /* Suscepts added */
   int immune;         /* Immunities added */
   int absorb;		  /* Absorbs added */
   int timer;          /* How much time is left */
   sh_int ac;
   sh_int cha;
   sh_int con;
   sh_int damroll;
   sh_int dex;
   sh_int dodge;
   sh_int hit;
   sh_int hitroll;
   sh_int inte;
   sh_int lck;
   sh_int mana;
   sh_int move;
   sh_int parry;
   sh_int saving_breath;
   sh_int saving_para_petri;
   sh_int saving_poison_death;
   sh_int saving_spell_staff;
   sh_int saving_wand;
   sh_int str;
   sh_int tumble;
   sh_int wis;
   bool cast_allowed; /* Casting allowed whilst morphed */
};

struct morph_data
{
   MORPH_DATA *next;          /* Next morph file */
   MORPH_DATA *prev;          /* Previous morph file */
   char *damroll;
   char *deity;
   char *description;
   char *help;          /* What player sees for info on morph */
   char *hit;           /* Hitpoints added */
   char *hitroll;
   char *key_words;     /* Keywords added to your name */
   char *long_desc;     /* New long_desc for player */
   char *mana;          /* Mana added not for vamps */
   char *morph_other;   /* What others see when you morph */
   char *morph_self;    /* What you see when you morph */
   char *move;          /* Move added */
   char *name;          /* Name used to polymorph into this */
   char *short_desc;    /* New short desc for player */
   char *no_skills;     /* Prevented Skills */
   char *skills;
   char *unmorph_other; /* What others see when you unmorph */
   char *unmorph_self;  /* What you see when you unmorph */
   EXT_BV affected_by;  /* New affected_by added */
   int Class;           /* Classes not allowed to use this */
   int	defpos;          /* Default position */
   EXT_BV no_affected_by;	/* Prevents affects from being added */
   int no_immune;	/* Prevents Immunities */
   int no_resistant;	/* Prevents resistances */
   int no_suscept;	/* Prevents Susceptibilities */
   int immune;		/* Immunities added */
   int obj[3];		/* Object needed to morph you */
   int race;           /* Races not allowed to use this */
   int resistant;      /* Resistances added */
   int suscept;        /* Suscepts added */
   int	absorb;		  /* Absorbs added - Samson 3-16-00 */
   int	timer;		/* Timer for how long it lasts */
   int used;           /* How many times has this morph been used */
   int vnum;           /* Unique identifier */
   sh_int ac;
   sh_int cha;		/* Amount Cha gained/Lost */
   sh_int con;		/* Amount of Con gained/Lost */
   sh_int dayfrom;		/* Starting Day you can morph into this */
   sh_int dayto;		/* Ending Day you can morph into this */
   sh_int dex;		/* Amount of dex added */
   sh_int dodge;		/* Percent of dodge added IE 1 = 1% */
   sh_int favourused;	/* Amount of favour to morph */
   sh_int hpused;		/* Amount of hps used to morph */
   sh_int inte;		/* Amount of Int gained/lost */
   sh_int lck;		/* Amount of Lck gained/lost */
   sh_int level;         /* Minimum level to use this morph */
   sh_int manaused;	/* Amount of mana used to morph */
   sh_int moveused;	/* Amount of move used to morph */
   sh_int parry;		/* Percent of parry added IE 1 = 1% */
   sh_int pkill;		/* Pkill Only, Peacefull Only or Both */
   sh_int saving_breath;	/* Below are saving adjusted */
   sh_int saving_para_petri;
   sh_int saving_poison_death;
   sh_int saving_spell_staff;
   sh_int saving_wand;
   sh_int sex;		/* The sex that can morph into this */
   sh_int str;		/* Amount of str gained lost */
   sh_int timefrom;	/* Hour starting you can morph */
   sh_int timeto;		/* Hour ending that you can morph */
   sh_int tumble;		/* Percent of tumble added IE 1 = 1% */
   sh_int wis;		/* Amount of Wis gained/lost */
   bool no_cast;	/* Can you cast a spell to morph into it */
   bool cast_allowed; /* Can you cast spells whilst morphed into this */
   bool objuse[3];	/* Objects needed to morph */
};
